"""


BMesh Operators (bmesh.ops)
***************************

This module gives access to low level bmesh operations.

Most operators take input and return output, they can be chained together
to perform useful operations.


Operator Example
================

This script shows how operators can be used to model a link of a chain.

.. code::

  # This script uses bmesh operators to make 2 links of a chain.

  import bpy
  import bmesh
  import math
  import mathutils

  # Make a new BMesh
  bm = bmesh.new()

  # Add a circle XXX, should return all geometry created, not just verts.
  bmesh.ops.create_circle(
      bm,
      cap_ends=False,
      radius=0.2,
      segments=8)


  # Spin and deal with geometry on side 'a'
  edges_start_a = bm.edges[:]
  geom_start_a = bm.verts[:] + edges_start_a
  ret = bmesh.ops.spin(
      bm,
      geom=geom_start_a,
      angle=math.radians(180.0),
      steps=8,
      axis=(1.0, 0.0, 0.0),
      cent=(0.0, 1.0, 0.0))
  edges_end_a = [ele for ele in ret["geom_last"]
                 if isinstance(ele, bmesh.types.BMEdge)]
  del ret


  # Extrude and create geometry on side 'b'
  ret = bmesh.ops.extrude_edge_only(
      bm,
      edges=edges_start_a)
  geom_extrude_mid = ret["geom"]
  del ret


  # Collect the edges to spin XXX, 'extrude_edge_only' could return this.
  verts_extrude_b = [ele for ele in geom_extrude_mid
                     if isinstance(ele, bmesh.types.BMVert)]
  edges_extrude_b = [ele for ele in geom_extrude_mid
                     if isinstance(ele, bmesh.types.BMEdge) and ele.is_boundary]
  bmesh.ops.translate(
      bm,
      verts=verts_extrude_b,
      vec=(0.0, 0.0, 1.0))


  # Create the circle on side 'b'
  ret = bmesh.ops.spin(
      bm,
      geom=verts_extrude_b + edges_extrude_b,
      angle=-math.radians(180.0),
      steps=8,
      axis=(1.0, 0.0, 0.0),
      cent=(0.0, 1.0, 1.0))
  edges_end_b = [ele for ele in ret["geom_last"]
                 if isinstance(ele, bmesh.types.BMEdge)]
  del ret


  # Bridge the resulting edge loops of both spins 'a & b'
  bmesh.ops.bridge_loops(
      bm,
      edges=edges_end_a + edges_end_b)


  # Now we have made a links of the chain, make a copy and rotate it
  # (so this looks something like a chain)

  ret = bmesh.ops.duplicate(
      bm,
      geom=bm.verts[:] + bm.edges[:] + bm.faces[:])
  geom_dupe = ret["geom"]
  verts_dupe = [ele for ele in geom_dupe if isinstance(ele, bmesh.types.BMVert)]
  del ret

  # position the new link
  bmesh.ops.translate(
      bm,
      verts=verts_dupe,
      vec=(0.0, 0.0, 2.0))
  bmesh.ops.rotate(
      bm,
      verts=verts_dupe,
      cent=(0.0, 1.0, 0.0),
      matrix=mathutils.Matrix.Rotation(math.radians(90.0), 3, 'Z'))

  # Done with creating the mesh, simply link it into the scene so we can see it

  # Finish up, write the bmesh into a new mesh
  me = bpy.data.meshes.new("Mesh")
  bm.to_mesh(me)
  bm.free()


  # Add the mesh to the scene
  obj = bpy.data.objects.new("Object", me)
  bpy.context.collection.objects.link(obj)

  # Select and make active
  bpy.context.view_layer.objects.active = obj
  obj.select_set(True)

:func:`smooth_vert`

:func:`smooth_laplacian_vert`

:func:`recalc_face_normals`

:func:`planar_faces`

:func:`region_extend`

:func:`rotate_edges`

:func:`reverse_faces`

:func:`bisect_edges`

:func:`mirror`

:func:`find_doubles`

:func:`remove_doubles`

:func:`collapse`

:func:`pointmerge_facedata`

:func:`average_vert_facedata`

:func:`pointmerge`

:func:`collapse_uvs`

:func:`weld_verts`

:func:`create_vert`

:func:`join_triangles`

:func:`contextual_create`

:func:`bridge_loops`

:func:`grid_fill`

:func:`holes_fill`

:func:`face_attribute_fill`

:func:`edgeloop_fill`

:func:`edgenet_fill`

:func:`edgenet_prepare`

:func:`rotate`

:func:`translate`

:func:`scale`

:func:`transform`

:func:`object_load_bmesh`

:func:`bmesh_to_mesh`

:func:`mesh_to_bmesh`

:func:`extrude_discrete_faces`

:func:`extrude_edge_only`

:func:`extrude_vert_indiv`

:func:`connect_verts`

:func:`connect_verts_concave`

:func:`connect_verts_nonplanar`

:func:`connect_vert_pair`

:func:`extrude_face_region`

:func:`dissolve_verts`

:func:`dissolve_edges`

:func:`dissolve_faces`

:func:`dissolve_limit`

:func:`dissolve_degenerate`

:func:`triangulate`

:func:`unsubdivide`

:func:`subdivide_edges`

:func:`subdivide_edgering`

:func:`bisect_plane`

:func:`delete`

:func:`duplicate`

:func:`split`

:func:`spin`

:func:`rotate_uvs`

:func:`reverse_uvs`

:func:`rotate_colors`

:func:`reverse_colors`

:func:`split_edges`

:func:`create_grid`

:func:`create_uvsphere`

:func:`create_icosphere`

:func:`create_monkey`

:func:`create_cone`

:func:`create_circle`

:func:`create_cube`

:func:`bevel`

:func:`beautify_fill`

:func:`triangle_fill`

:func:`solidify`

:func:`inset_individual`

:func:`inset_region`

:func:`offset_edgeloops`

:func:`wireframe`

:func:`poke`

:func:`convex_hull`

:func:`symmetrize`

"""

import typing

import mathutils

import bpy

import bmesh

def smooth_vert(bm: bmesh.types.BMesh, verts: typing.List[typing.Any], factor: float, mirror_clip_x: bool, mirror_clip_y: bool, mirror_clip_z: bool, clip_dist: float, use_axis_x: bool, use_axis_y: bool, use_axis_z: bool) -> None:

  """

  Vertex Smooth.

  Smooths vertices by using a basic vertex averaging scheme.

  """

  ...

def smooth_laplacian_vert(bm: bmesh.types.BMesh, verts: typing.List[typing.Any], lambda_factor: float, lambda_border: float, use_x: bool, use_y: bool, use_z: bool, preserve_volume: bool) -> None:

  """

  Vertex Smooth Laplacian.

  Smooths vertices by using Laplacian smoothing propose by.
Desbrun, et al. Implicit Fairing of Irregular Meshes using Diffusion and Curvature Flow.

  """

  ...

def recalc_face_normals(bm: bmesh.types.BMesh, faces: typing.List[typing.Any]) -> None:

  """

  Right-Hand Faces.

  Computes an "outside" normal for the specified input faces.

  """

  ...

def planar_faces(bm: bmesh.types.BMesh, faces: typing.List[typing.Any], iterations: int, factor: float) -> typing.Dict[str, typing.Any]:

  """

  Planar Faces.

  Iteratively flatten faces.

  """

  ...

def region_extend(bm: bmesh.types.BMesh, geom: typing.List[typing.Any], use_contract: bool, use_faces: bool, use_face_step: bool) -> typing.Dict[str, typing.Any]:

  """

  Region Extend.

  used to implement the select more/less tools.
this puts some geometry surrounding regions of
geometry in geom into geom.out.

  if use_faces is 0 then geom.out spits out verts and edges,
otherwise it spits out faces.

  """

  ...

def rotate_edges(bm: bmesh.types.BMesh, edges: typing.List[typing.Any], use_ccw: bool) -> typing.Dict[str, typing.Any]:

  """

  Edge Rotate.

  Rotates edges topologically.  Also known as "spin edge" to some people.
Simple example: ``[/] becomes [|] then [\]``.

  """

  ...

def reverse_faces(bm: bmesh.types.BMesh, faces: typing.List[typing.Any], flip_multires: bool) -> None:

  """

  Reverse Faces.

  Reverses the winding (vertex order) of faces.
This has the effect of flipping the normal.

  """

  ...

def bisect_edges(bm: bmesh.types.BMesh, edges: typing.List[typing.Any], cuts: int, edge_percents: typing.Dict[str, typing.Any]) -> typing.Dict[str, typing.Any]:

  """

  Edge Bisect.

  Splits input edges (but doesn't do anything else).
This creates a 2-valence vert.

  """

  ...

def mirror(bm: bmesh.types.BMesh, geom: typing.List[typing.Any], matrix: mathutils.Matrix, merge_dist: float, axis: str, mirror_u: bool, mirror_v: bool, mirror_udim: bool) -> typing.Dict[str, typing.Any]:

  """

  Mirror.

  Mirrors geometry along an axis.  The resulting geometry is welded on using
merge_dist.  Pairs of original/mirrored vertices are welded using the merge_dist
parameter (which defines the minimum distance for welding to happen).

  """

  ...

def find_doubles(bm: bmesh.types.BMesh, verts: typing.List[typing.Any], keep_verts: typing.List[typing.Any], dist: float) -> typing.Dict[str, typing.Any]:

  """

  Find Doubles.

  Takes input verts and find vertices they should weld to.
Outputs a mapping slot suitable for use with the weld verts bmop.

  If keep_verts is used, vertices outside that set can only be merged
with vertices in that set.

  """

  ...

def remove_doubles(bm: bmesh.types.BMesh, verts: typing.List[typing.Any], dist: float) -> None:

  """

  Remove Doubles.

  Finds groups of vertices closer than dist and merges them together,
using the weld verts bmop.

  """

  ...

def collapse(bm: bmesh.types.BMesh, edges: typing.List[typing.Any], uvs: bool) -> None:

  """

  Collapse Connected.

  Collapses connected vertices

  """

  ...

def pointmerge_facedata(bm: bmesh.types.BMesh, verts: typing.List[typing.Any], vert_snap: bmesh.types.BMVert) -> None:

  """

  Face-Data Point Merge.

  Merge uv/vcols at a specific vertex.

  """

  ...

def average_vert_facedata(bm: bmesh.types.BMesh, verts: typing.List[typing.Any]) -> None:

  """

  Average Vertices Facevert Data.

  Merge uv/vcols associated with the input vertices at
the bounding box center. (I know, it's not averaging but
the vert_snap_to_bb_center is just too long).

  """

  ...

def pointmerge(bm: bmesh.types.BMesh, verts: typing.List[typing.Any], merge_co: typing.Union[mathutils.Vector, typing.Any]) -> None:

  """

  Point Merge.

  Merge verts together at a point.

  """

  ...

def collapse_uvs(bm: bmesh.types.BMesh, edges: typing.List[typing.Any]) -> None:

  """

  Collapse Connected UV's.

  Collapses connected UV vertices.

  """

  ...

def weld_verts(bm: bmesh.types.BMesh, targetmap: typing.Dict[str, typing.Any]) -> None:

  """

  Weld Verts.

  Welds verts together (kind-of like remove doubles, merge, etc, all of which
use or will use this bmop).  You pass in mappings from vertices to the vertices
they weld with.

  """

  ...

def create_vert(bm: bmesh.types.BMesh, co: typing.Union[mathutils.Vector, typing.Any]) -> typing.Dict[str, typing.Any]:

  """

  Make Vertex.

  Creates a single vertex; this bmop was necessary
for click-create-vertex.

  """

  ...

def join_triangles(bm: bmesh.types.BMesh, faces: typing.List[typing.Any], cmp_seam: bool, cmp_sharp: bool, cmp_uvs: bool, cmp_vcols: bool, cmp_materials: bool, angle_face_threshold: float, angle_shape_threshold: float) -> typing.Dict[str, typing.Any]:

  """

  Join Triangles.

  Tries to intelligently join triangles according
to angle threshold and delimiters.

  """

  ...

def contextual_create(bm: bmesh.types.BMesh, geom: typing.List[typing.Any], mat_nr: int, use_smooth: bool) -> typing.Dict[str, typing.Any]:

  """

  Contextual Create.

  This is basically F-key, it creates
new faces from vertices, makes stuff from edge nets,
makes wire edges, etc.  It also dissolves faces.

  Three verts become a triangle, four become a quad.  Two
become a wire edge.

  """

  ...

def bridge_loops(bm: bmesh.types.BMesh, edges: typing.List[typing.Any], use_pairs: bool, use_cyclic: bool, use_merge: bool, merge_factor: float, twist_offset: int) -> typing.Dict[str, typing.Any]:

  """

  Bridge edge loops with faces.

  """

  ...

def grid_fill(bm: bmesh.types.BMesh, edges: typing.List[typing.Any], mat_nr: int, use_smooth: bool, use_interp_simple: bool) -> typing.Dict[str, typing.Any]:

  """

  Grid Fill.

  Create faces defined by 2 disconnected edge loops (which share edges).

  """

  ...

def holes_fill(bm: bmesh.types.BMesh, edges: typing.List[typing.Any], sides: int) -> typing.Dict[str, typing.Any]:

  """

  Fill Holes.

  Fill boundary edges with faces, copying surrounding customdata.

  """

  ...

def face_attribute_fill(bm: bmesh.types.BMesh, faces: typing.List[typing.Any], use_normals: bool, use_data: bool) -> typing.Dict[str, typing.Any]:

  """

  Face Attribute Fill.

  Fill in faces with data from adjacent faces.

  """

  ...

def edgeloop_fill(bm: bmesh.types.BMesh, edges: typing.List[typing.Any], mat_nr: int, use_smooth: bool) -> typing.Dict[str, typing.Any]:

  """

  Edge Loop Fill.

  Create faces defined by one or more non overlapping edge loops.

  """

  ...

def edgenet_fill(bm: bmesh.types.BMesh, edges: typing.List[typing.Any], mat_nr: int, use_smooth: bool, sides: int) -> typing.Dict[str, typing.Any]:

  """

  Edge Net Fill.

  Create faces defined by enclosed edges.

  """

  ...

def edgenet_prepare(bm: bmesh.types.BMesh, edges: typing.List[typing.Any]) -> typing.Dict[str, typing.Any]:

  """

  Edgenet Prepare.

  Identifies several useful edge loop cases and modifies them so
they'll become a face when edgenet_fill is called.  The cases covered are:

  * One single loop; an edge is added to connect the ends

  * Two loops; two edges are added to connect the endpoints (based on the
shortest distance between each endpont).

  """

  ...

def rotate(bm: bmesh.types.BMesh, cent: typing.Union[mathutils.Vector, typing.Any], matrix: mathutils.Matrix, verts: typing.List[typing.Any], space: mathutils.Matrix) -> None:

  """

  Rotate.

  Rotate vertices around a center, using a 3x3 rotation matrix.

  """

  ...

def translate(bm: bmesh.types.BMesh, vec: typing.Union[mathutils.Vector, typing.Any], space: mathutils.Matrix, verts: typing.List[typing.Any]) -> None:

  """

  Translate.

  Translate vertices by an offset.

  """

  ...

def scale(bm: bmesh.types.BMesh, vec: typing.Union[mathutils.Vector, typing.Any], space: mathutils.Matrix, verts: typing.List[typing.Any]) -> None:

  """

  Scale.

  Scales vertices by an offset.

  """

  ...

def transform(bm: bmesh.types.BMesh, matrix: mathutils.Matrix, space: mathutils.Matrix, verts: typing.List[typing.Any]) -> None:

  """

  Transform.

  Transforms a set of vertices by a matrix.  Multiplies
the vertex coordinates with the matrix.

  """

  ...

def object_load_bmesh(bm: bmesh.types.BMesh, scene: bpy.types.Scene, object: bpy.types.Object) -> None:

  """

  Object Load BMesh.

  Loads a bmesh into an object/mesh.  This is a "private"
bmop.

  """

  ...

def bmesh_to_mesh(bm: bmesh.types.BMesh, mesh: bpy.types.Mesh, object: bpy.types.Object) -> None:

  """

  BMesh to Mesh.

  Converts a bmesh to a Mesh.  This is reserved for exiting editmode.

  """

  ...

def mesh_to_bmesh(bm: bmesh.types.BMesh, mesh: bpy.types.Mesh, object: bpy.types.Object, use_shapekey: bool) -> None:

  """

  Mesh to BMesh.

  Load the contents of a mesh into the bmesh.  this bmop is private, it's
reserved exclusively for entering editmode.

  """

  ...

def extrude_discrete_faces(bm: bmesh.types.BMesh, faces: typing.List[typing.Any], use_normal_flip: bool, use_select_history: bool) -> typing.Dict[str, typing.Any]:

  """

  Individual Face Extrude.

  Extrudes faces individually.

  """

  ...

def extrude_edge_only(bm: bmesh.types.BMesh, edges: typing.List[typing.Any], use_normal_flip: bool, use_select_history: bool) -> typing.Dict[str, typing.Any]:

  """

  Extrude Only Edges.

  Extrudes Edges into faces, note that this is very simple, there's no fancy
winged extrusion.

  """

  ...

def extrude_vert_indiv(bm: bmesh.types.BMesh, verts: typing.List[typing.Any], use_select_history: bool) -> typing.Dict[str, typing.Any]:

  """

  Individual Vertex Extrude.

  Extrudes wire edges from vertices.

  """

  ...

def connect_verts(bm: bmesh.types.BMesh, verts: typing.List[typing.Any], faces_exclude: typing.List[typing.Any], check_degenerate: bool) -> typing.Dict[str, typing.Any]:

  """

  Connect Verts.

  Split faces by adding edges that connect **verts**.

  """

  ...

def connect_verts_concave(bm: bmesh.types.BMesh, faces: typing.List[typing.Any]) -> typing.Dict[str, typing.Any]:

  """

  Connect Verts to form Convex Faces.

  Ensures all faces are convex **faces**.

  """

  ...

def connect_verts_nonplanar(bm: bmesh.types.BMesh, angle_limit: float, faces: typing.List[typing.Any]) -> typing.Dict[str, typing.Any]:

  """

  Connect Verts Across non Planer Faces.

  Split faces by connecting edges along non planer **faces**.

  """

  ...

def connect_vert_pair(bm: bmesh.types.BMesh, verts: typing.List[typing.Any], verts_exclude: typing.List[typing.Any], faces_exclude: typing.List[typing.Any]) -> typing.Dict[str, typing.Any]:

  """

  Connect Verts.

  Split faces by adding edges that connect **verts**.

  """

  ...

def extrude_face_region(bm: bmesh.types.BMesh, geom: typing.List[typing.Any], edges_exclude: typing.Set[typing.Any], use_keep_orig: bool, use_normal_flip: bool, use_normal_from_adjacent: bool, use_dissolve_ortho_edges: bool, use_select_history: bool) -> typing.Dict[str, typing.Any]:

  """

  Extrude Faces.

  Extrude operator (does not transform)

  """

  ...

def dissolve_verts(bm: bmesh.types.BMesh, verts: typing.List[typing.Any], use_face_split: bool, use_boundary_tear: bool) -> None:

  """

  Dissolve Verts.

  """

  ...

def dissolve_edges(bm: bmesh.types.BMesh, edges: typing.List[typing.Any], use_verts: bool, use_face_split: bool) -> typing.Dict[str, typing.Any]:

  """

  Dissolve Edges.

  """

  ...

def dissolve_faces(bm: bmesh.types.BMesh, faces: typing.List[typing.Any], use_verts: bool) -> typing.Dict[str, typing.Any]:

  """

  Dissolve Faces.

  """

  ...

def dissolve_limit(bm: bmesh.types.BMesh, angle_limit: float, use_dissolve_boundaries: bool, verts: typing.List[typing.Any], edges: typing.List[typing.Any], delimit: typing.Set[typing.Any]) -> typing.Dict[str, typing.Any]:

  """

  Limited Dissolve.

  Dissolve planar faces and co-linear edges.

  """

  ...

def dissolve_degenerate(bm: bmesh.types.BMesh, dist: float, edges: typing.List[typing.Any]) -> None:

  """

  Degenerate Dissolve.

  Dissolve edges with no length, faces with no area.

  """

  ...

def triangulate(bm: bmesh.types.BMesh, faces: typing.List[typing.Any], quad_method: str, ngon_method: str) -> typing.Dict[str, typing.Any]:

  """

  Triangulate.

  """

  ...

def unsubdivide(bm: bmesh.types.BMesh, verts: typing.List[typing.Any], iterations: int) -> None:

  """

  Un-Subdivide.

  Reduce detail in geometry containing grids.

  """

  ...

def subdivide_edges(bm: bmesh.types.BMesh, edges: typing.List[typing.Any], smooth: float, smooth_falloff: str, fractal: float, along_normal: float, cuts: int, seed: int, custom_patterns: typing.Dict[str, typing.Any], edge_percents: typing.Dict[str, typing.Any], quad_corner_type: str, use_grid_fill: bool, use_single_edge: bool, use_only_quads: bool, use_sphere: bool, use_smooth_even: bool) -> typing.Dict[str, typing.Any]:

  """

  Subdivide Edges.

  Advanced operator for subdividing edges
with options for face patterns, smoothing and randomization.

  """

  ...

def subdivide_edgering(bm: bmesh.types.BMesh, edges: typing.List[typing.Any], interp_mode: str, smooth: float, cuts: int, profile_shape: str, profile_shape_factor: float) -> typing.Dict[str, typing.Any]:

  """

  Subdivide Edge-Ring.

  Take an edge-ring, and subdivide with interpolation options.

  """

  ...

def bisect_plane(bm: bmesh.types.BMesh, geom: typing.List[typing.Any], dist: float, plane_co: typing.Union[mathutils.Vector, typing.Any], plane_no: typing.Union[mathutils.Vector, typing.Any], use_snap_center: bool, clear_outer: bool, clear_inner: bool) -> typing.Dict[str, typing.Any]:

  """

  Bisect Plane.

  Bisects the mesh by a plane (cut the mesh in half).

  """

  ...

def delete(bm: bmesh.types.BMesh, geom: typing.List[typing.Any], context: str) -> None:

  """

  Delete Geometry.

  Utility operator to delete geometry.

  """

  ...

def duplicate(bm: bmesh.types.BMesh, geom: typing.List[typing.Any], dest: bmesh.types.BMesh, use_select_history: bool, use_edge_flip_from_face: bool) -> typing.Dict[str, typing.Any]:

  """

  Duplicate Geometry.

  Utility operator to duplicate geometry,
optionally into a destination mesh.

  """

  ...

def split(bm: bmesh.types.BMesh, geom: typing.List[typing.Any], dest: bmesh.types.BMesh, use_only_faces: bool) -> typing.Dict[str, typing.Any]:

  """

  Split Off Geometry.

  Disconnect geometry from adjacent edges and faces,
optionally into a destination mesh.

  """

  ...

def spin(bm: bmesh.types.BMesh, geom: typing.List[typing.Any], cent: typing.Union[mathutils.Vector, typing.Any], axis: typing.Union[mathutils.Vector, typing.Any], dvec: typing.Union[mathutils.Vector, typing.Any], angle: float, space: mathutils.Matrix, steps: int, use_merge: bool, use_normal_flip: bool, use_duplicate: bool) -> typing.Dict[str, typing.Any]:

  """

  Spin.

  Extrude or duplicate geometry a number of times,
rotating and possibly translating after each step

  """

  ...

def rotate_uvs(bm: bmesh.types.BMesh, faces: typing.List[typing.Any], use_ccw: bool) -> None:

  """

  UV Rotation.

  Cycle the loop UV's

  """

  ...

def reverse_uvs(bm: bmesh.types.BMesh, faces: typing.List[typing.Any]) -> None:

  """

  UV Reverse.

  Reverse the UV's

  """

  ...

def rotate_colors(bm: bmesh.types.BMesh, faces: typing.List[typing.Any], use_ccw: bool) -> None:

  """

  Color Rotation.

  Cycle the loop colors

  """

  ...

def reverse_colors(bm: bmesh.types.BMesh, faces: typing.List[typing.Any]) -> None:

  """

  Color Reverse

  Reverse the loop colors.

  """

  ...

def split_edges(bm: bmesh.types.BMesh, edges: typing.List[typing.Any], verts: typing.List[typing.Any], use_verts: bool) -> typing.Dict[str, typing.Any]:

  """

  Edge Split.

  Disconnects faces along input edges.

  """

  ...

def create_grid(bm: bmesh.types.BMesh, x_segments: int, y_segments: int, size: float, matrix: mathutils.Matrix, calc_uvs: bool) -> typing.Dict[str, typing.Any]:

  """

  Create Grid.

  Creates a grid with a variable number of subdivisions

  """

  ...

def create_uvsphere(bm: bmesh.types.BMesh, u_segments: int, v_segments: int, diameter: float, matrix: mathutils.Matrix, calc_uvs: bool) -> typing.Dict[str, typing.Any]:

  """

  Create UV Sphere.

  Creates a grid with a variable number of subdivisions

  """

  ...

def create_icosphere(bm: bmesh.types.BMesh, subdivisions: int, diameter: float, matrix: mathutils.Matrix, calc_uvs: bool) -> typing.Dict[str, typing.Any]:

  """

  Create Ico-Sphere.

  Creates a grid with a variable number of subdivisions

  """

  ...

def create_monkey(bm: bmesh.types.BMesh, matrix: mathutils.Matrix, calc_uvs: bool) -> typing.Dict[str, typing.Any]:

  """

  Create Suzanne.

  Creates a monkey (standard blender primitive).

  """

  ...

def create_cone(bm: bmesh.types.BMesh, cap_ends: bool, cap_tris: bool, segments: int, diameter1: float, diameter2: float, depth: float, matrix: mathutils.Matrix, calc_uvs: bool) -> typing.Dict[str, typing.Any]:

  """

  Create Cone.

  Creates a cone with variable depth at both ends

  """

  ...

def create_circle(bm: bmesh.types.BMesh, cap_ends: bool, cap_tris: bool, segments: int, radius: float, matrix: mathutils.Matrix, calc_uvs: bool) -> typing.Dict[str, typing.Any]:

  """

  Creates a Circle.

  """

  ...

def create_cube(bm: bmesh.types.BMesh, size: float, matrix: mathutils.Matrix, calc_uvs: bool) -> typing.Dict[str, typing.Any]:

  """

  Create Cube

  Creates a cube.

  """

  ...

def bevel(bm: bmesh.types.BMesh, geom: typing.List[typing.Any], offset: float, offset_type: str, profile_type: str, segments: int, profile: float, affect: str, clamp_overlap: bool, material: int, loop_slide: bool, mark_seam: bool, mark_sharp: bool, harden_normals: bool, face_strength_mode: str, miter_outer: str, miter_inner: str, spread: float, smoothresh: float, custom_profile: bpy.types.bpy_struct, vmesh_method: str) -> typing.Dict[str, typing.Any]:

  """

  Bevel.

  Bevels edges and vertices

  """

  ...

def beautify_fill(bm: bmesh.types.BMesh, faces: typing.List[typing.Any], edges: typing.List[typing.Any], use_restrict_tag: bool, method: str) -> typing.Dict[str, typing.Any]:

  """

  Beautify Fill.

  Rotate edges to create more evenly spaced triangles.

  """

  ...

def triangle_fill(bm: bmesh.types.BMesh, use_beauty: bool, use_dissolve: bool, edges: typing.List[typing.Any], normal: typing.Union[mathutils.Vector, typing.Any]) -> typing.Dict[str, typing.Any]:

  """

  Triangle Fill.

  Fill edges with triangles

  """

  ...

def solidify(bm: bmesh.types.BMesh, geom: typing.List[typing.Any], thickness: float) -> typing.Dict[str, typing.Any]:

  """

  Solidify.

  Turns a mesh into a shell with thickness

  """

  ...

def inset_individual(bm: bmesh.types.BMesh, faces: typing.List[typing.Any], thickness: float, depth: float, use_even_offset: bool, use_interpolate: bool, use_relative_offset: bool) -> typing.Dict[str, typing.Any]:

  """

  Face Inset (Individual).

  Insets individual faces.

  """

  ...

def inset_region(bm: bmesh.types.BMesh, faces: typing.List[typing.Any], faces_exclude: typing.List[typing.Any], use_boundary: bool, use_even_offset: bool, use_interpolate: bool, use_relative_offset: bool, use_edge_rail: bool, thickness: float, depth: float, use_outset: bool) -> typing.Dict[str, typing.Any]:

  """

  Face Inset (Regions).

  Inset or outset face regions.

  """

  ...

def offset_edgeloops(bm: bmesh.types.BMesh, edges: typing.List[typing.Any], use_cap_endpoint: bool) -> typing.Dict[str, typing.Any]:

  """

  Edgeloop Offset.

  Creates edge loops based on simple edge-outset method.

  """

  ...

def wireframe(bm: bmesh.types.BMesh, faces: typing.List[typing.Any], thickness: float, offset: float, use_replace: bool, use_boundary: bool, use_even_offset: bool, use_crease: bool, crease_weight: float, use_relative_offset: bool, material_offset: int) -> typing.Dict[str, typing.Any]:

  """

  Wire Frame.

  Makes a wire-frame copy of faces.

  """

  ...

def poke(bm: bmesh.types.BMesh, faces: typing.List[typing.Any], offset: float, center_mode: str, use_relative_offset: bool) -> typing.Dict[str, typing.Any]:

  """

  Pokes a face.

  Splits a face into a triangle fan.

  """

  ...

def convex_hull(bm: bmesh.types.BMesh, input: typing.List[typing.Any], use_existing_faces: bool) -> typing.Dict[str, typing.Any]:

  """

  Convex Hull

  Builds a convex hull from the vertices in 'input'.

  If 'use_existing_faces' is true, the hull will not output triangles
that are covered by a pre-existing face.

  All hull vertices, faces, and edges are added to 'geom.out'. Any
input elements that end up inside the hull (i.e. are not used by an
output face) are added to the 'interior_geom' slot. The
'unused_geom' slot will contain all interior geometry that is
completely unused. Lastly, 'holes_geom' contains edges and faces
that were in the input and are part of the hull.

  """

  ...

def symmetrize(bm: bmesh.types.BMesh, input: typing.List[typing.Any], direction: str, dist: float) -> typing.Dict[str, typing.Any]:

  """

  Symmetrize.

  Makes the mesh elements in the "input" slot symmetrical. Unlike
normal mirroring, it only copies in one direction, as specified by
the "direction" slot. The edges and faces that cross the plane of
symmetry are split as needed to enforce symmetry.

  All new vertices, edges, and faces are added to the "geom.out" slot.

  """

  ...



"""


Geometry Utilities (mathutils.geometry)
***************************************

The Blender geometry module

:func:`area_tri`

:func:`barycentric_transform`

:func:`box_fit_2d`

:func:`box_pack_2d`

:func:`closest_point_on_tri`

:func:`convex_hull_2d`

:func:`delaunay_2d_cdt`

:func:`distance_point_to_plane`

:func:`interpolate_bezier`

:func:`intersect_line_line`

:func:`intersect_line_line_2d`

:func:`intersect_line_plane`

:func:`intersect_line_sphere`

:func:`intersect_line_sphere_2d`

:func:`intersect_plane_plane`

:func:`intersect_point_line`

:func:`intersect_point_quad_2d`

:func:`intersect_point_tri`

:func:`intersect_point_tri_2d`

:func:`intersect_ray_tri`

:func:`intersect_sphere_sphere_2d`

:func:`intersect_tri_tri_2d`

:func:`normal`

:func:`points_in_planes`

:func:`tessellate_polygon`

:func:`volume_tetrahedron`

"""

import typing

import mathutils

def area_tri(v1: mathutils.Vector, v2: mathutils.Vector, v3: mathutils.Vector) -> float:

  """

  Returns the area size of the 2D or 3D triangle defined.

  """

  ...

def barycentric_transform(point: mathutils.Vector, tri_a1: mathutils.Vector, tri_a2: mathutils.Vector, tri_a3: mathutils.Vector, tri_b1: mathutils.Vector, tri_b2: mathutils.Vector, tri_b3: mathutils.Vector) -> typing.Any:

  """

  Return a transformed point, the transformation is defined by 2 triangles.

  """

  ...

def box_fit_2d(points: typing.List[typing.Any]) -> float:

  """

  Returns an angle that best fits the points to an axis aligned rectangle

  """

  ...

def box_pack_2d(boxes: typing.List[typing.Any]) -> typing.Tuple[typing.Any, ...]:

  """

  Returns a tuple with the width and height of the packed bounding box.

  """

  ...

def closest_point_on_tri(pt: mathutils.Vector, tri_p1: mathutils.Vector, tri_p2: mathutils.Vector, tri_p3: mathutils.Vector) -> mathutils.Vector:

  """

  Takes 4 vectors: one is the point and the next 3 define the triangle.

  """

  ...

def convex_hull_2d(points: typing.List[typing.Any]) -> typing.List[int]:

  """

  Returns a list of indices into the list given

  """

  ...

def delaunay_2d_cdt(vert_coords: typing.List[mathutils.Vector], edges: typing.List[typing.Any], faces: typing.List[typing.List[typing.Any]], output_type: typing.Any, epsilon: float) -> typing.Any:

  """

  Computes the Constrained Delaunay Triangulation of a set of vertices,
with edges and faces that must appear in the triangulation.
Some triangles may be eaten away, or combined with other triangles,
according to output type.
The returned verts may be in a different order from input verts, may be moved
slightly, and may be merged with other nearby verts.
The three returned orig lists give, for each of verts, edges, and faces, the list of
input element indices corresponding to the positionally same output element.
For edges, the orig indices start with the input edges and then continue
with the edges implied by each of the faces (n of them for an n-gon).

  """

  ...

def distance_point_to_plane(pt: mathutils.Vector, plane_co: mathutils.Vector, plane_no: mathutils.Vector) -> float:

  """

  Returns the signed distance between a point and a plane    (negative when below the normal).

  """

  ...

def interpolate_bezier(knot1: mathutils.Vector, handle1: mathutils.Vector, handle2: mathutils.Vector, knot2: mathutils.Vector, resolution: int) -> typing.List[mathutils.Vector]:

  """

  Interpolate a bezier spline segment.

  """

  ...

def intersect_line_line(v1: mathutils.Vector, v2: mathutils.Vector, v3: mathutils.Vector, v4: mathutils.Vector) -> typing.Tuple[mathutils.Vector, ...]:

  """

  Returns a tuple with the points on each line respectively closest to the other.

  """

  ...

def intersect_line_line_2d(lineA_p1: mathutils.Vector, lineA_p2: mathutils.Vector, lineB_p1: mathutils.Vector, lineB_p2: mathutils.Vector) -> mathutils.Vector:

  """

  Takes 2 segments (defined by 4 vectors) and returns a vector for their point of intersection or None.

  Warning: Despite its name, this function works on segments, and not on lines.

  """

  ...

def intersect_line_plane(line_a: mathutils.Vector, line_b: mathutils.Vector, plane_co: mathutils.Vector, plane_no: mathutils.Vector, no_flip: typing.Any = False) -> mathutils.Vector:

  """

  Calculate the intersection between a line (as 2 vectors) and a plane.
Returns a vector for the intersection or None.

  """

  ...

def intersect_line_sphere(line_a: mathutils.Vector, line_b: mathutils.Vector, sphere_co: mathutils.Vector, sphere_radius: typing.Any, clip: typing.Any = True) -> typing.Any:

  """

  Takes a line (as 2 points) and a sphere (as a point and a radius) and
returns the intersection

  """

  ...

def intersect_line_sphere_2d(line_a: mathutils.Vector, line_b: mathutils.Vector, sphere_co: mathutils.Vector, sphere_radius: typing.Any, clip: typing.Any = True) -> typing.Any:

  """

  Takes a line (as 2 points) and a sphere (as a point and a radius) and
returns the intersection

  """

  ...

def intersect_plane_plane(plane_a_co: mathutils.Vector, plane_a_no: mathutils.Vector, plane_b_co: mathutils.Vector, plane_b_no: mathutils.Vector) -> typing.Tuple[typing.Any, ...]:

  """

  Return the intersection between two planes

  """

  ...

def intersect_point_line(pt: mathutils.Vector, line_p1: mathutils.Vector, line_p2: typing.Any) -> typing.Any:

  """

  Takes a point and a line and returns a tuple with the closest point on the line and its distance from the first point of the line as a percentage of the length of the line.

  """

  ...

def intersect_point_quad_2d(pt: mathutils.Vector, quad_p1: mathutils.Vector, quad_p2: mathutils.Vector, quad_p3: mathutils.Vector, quad_p4: mathutils.Vector) -> int:

  """

  Takes 5 vectors (using only the x and y coordinates): one is the point and the next 4 define the quad,
only the x and y are used from the vectors. Returns 1 if the point is within the quad, otherwise 0.
Works only with convex quads without singular edges.

  """

  ...

def intersect_point_tri(pt: mathutils.Vector, tri_p1: mathutils.Vector, tri_p2: mathutils.Vector, tri_p3: mathutils.Vector) -> mathutils.Vector:

  """

  Takes 4 vectors: one is the point and the next 3 define the triangle. Projects the point onto the triangle plane and checks if it is within the triangle.

  """

  ...

def intersect_point_tri_2d(pt: mathutils.Vector, tri_p1: mathutils.Vector, tri_p2: mathutils.Vector, tri_p3: mathutils.Vector) -> int:

  """

  Takes 4 vectors (using only the x and y coordinates): one is the point and the next 3 define the triangle. Returns 1 if the point is within the triangle, otherwise 0.

  """

  ...

def intersect_ray_tri(v1: mathutils.Vector, v2: mathutils.Vector, v3: mathutils.Vector, ray: mathutils.Vector, orig: mathutils.Vector, clip: bool = True) -> mathutils.Vector:

  """

  Returns the intersection between a ray and a triangle, if possible, returns None otherwise.

  """

  ...

def intersect_sphere_sphere_2d(p_a: mathutils.Vector, radius_a: float, p_b: mathutils.Vector, radius_b: float) -> typing.Tuple[mathutils.Vector, ...]:

  """

  Returns 2 points on between intersecting circles.

  """

  ...

def intersect_tri_tri_2d(tri_a1: typing.Any, tri_a2: typing.Any, tri_a3: typing.Any, tri_b1: typing.Any, tri_b2: typing.Any, tri_b3: typing.Any) -> bool:

  """

  Check if two 2D triangles intersect.

  """

  ...

def normal(vectors: typing.Sequence[typing.Any]) -> mathutils.Vector:

  """

  Returns the normal of a 3D polygon.

  """

  ...

def points_in_planes(planes: typing.List[mathutils.Vector]) -> typing.Any:

  """

  Returns a list of points inside all planes given and a list of index values for the planes used.

  """

  ...

def tessellate_polygon(veclist_list: typing.Any) -> typing.List[typing.Any]:

  """

  Takes a list of polylines (each point a pair or triplet of numbers) and returns the point indices for a polyline filled with triangles. Does not handle degenerate geometry (such as zero-length lines due to consecutive identical points).

  """

  ...

def volume_tetrahedron(v1: mathutils.Vector, v2: mathutils.Vector, v3: mathutils.Vector, v4: mathutils.Vector) -> float:

  """

  Return the volume formed by a tetrahedron (points can be in any order).

  """

  ...



from . import utils

from . import types

from . import props

from . import path

from . import ops

from . import msgbus

from . import context

from . import app

import typing

data: types.BlendData = ...

"""

Access to Blender's internal data

"""



"""


gpu_extras submodule (gpu_extras.batch)
***************************************

:func:`batch_for_shader`

"""

import typing

import gpu

def batch_for_shader(shader: gpu.types.GPUShader, type: str, content: typing.Dict[str, typing.Any], indices: typing.Any = None) -> gpu.types.Batch:

  """

  Return a batch already configured and compatible with the shader.

  """

  ...



"""


bpy_extras submodule (bpy_extras.view3d_utils)
**********************************************

:func:`region_2d_to_vector_3d`

:func:`region_2d_to_origin_3d`

:func:`region_2d_to_location_3d`

:func:`location_3d_to_region_2d`

"""

import typing

import mathutils

import bpy

def region_2d_to_vector_3d(region: bpy.types.Region, rv3d: bpy.types.RegionView3D, coord: mathutils.Vector) -> mathutils.Vector:

  """

  Return a direction vector from the viewport at the specific 2d region
coordinate.

  """

  ...

def region_2d_to_origin_3d(region: bpy.types.Region, rv3d: bpy.types.RegionView3D, coord: mathutils.Vector, clamp: float = None) -> mathutils.Vector:

  """

  Return the 3d view origin from the region relative 2d coords.

  Note: Orthographic views have a less obvious origin,
the far clip is used to define the viewport near/far extents.
Since far clip can be a very large value,
the result may give with numeric precision issues.To avoid this problem, you can optionally clamp the far clip to a
smaller value based on the data you're operating on.

  """

  ...

def region_2d_to_location_3d(region: bpy.types.Region, rv3d: bpy.types.RegionView3D, coord: mathutils.Vector, depth_location: mathutils.Vector) -> mathutils.Vector:

  """

  Return a 3d location from the region relative 2d coords, aligned with
*depth_location*.

  """

  ...

def location_3d_to_region_2d(region: bpy.types.Region, rv3d: bpy.types.RegionView3D, coord: mathutils.Vector, default: typing.Any = None) -> mathutils.Vector:

  """

  Return the *region* relative 2d location of a 3d position.

  """

  ...



"""


Additional Math Functions (bl_math)
***********************************

Miscellaneous math utilities module

:func:`clamp`

"""

import typing

def clamp(value: float, min: float = 0, max: float = 1) -> float:

  """

  Clamps the float value between minimum and maximum. To avoid
confusion, any call must use either one or all three arguments.

  """

  ...



"""


Collection Operators
********************

:func:`create`

:func:`objects_add_active`

:func:`objects_remove`

:func:`objects_remove_active`

:func:`objects_remove_all`

"""

import typing

def create(name: str = 'Collection') -> None:

  """

  Create an object collection from selected objects

  """

  ...

def objects_add_active(collection: str = '') -> None:

  """

  Add the object to an object collection that contains the active object

  """

  ...

def objects_remove(collection: str = '') -> None:

  """

  Remove selected objects from a collection

  """

  ...

def objects_remove_active(collection: str = '') -> None:

  """

  Remove the object from an object collection that contains the active object

  """

  ...

def objects_remove_all() -> None:

  """

  Remove selected objects from all collections

  """

  ...



"""


GPU Shader (gpu.shader)
***********************

This module provides access to GPUShader internal functions.

-[ Built-in shaders ]-

All built-in shaders have the ``mat4 ModelViewProjectionMatrix`` uniform.
The value of it can only be modified using the :class:`gpu.matrix` module.

2D_UNIFORM_COLOR
  :Attributes:      
    vec3 pos

  :Uniforms:        
    vec4 color

2D_FLAT_COLOR
  :Attributes:      
    vec3 pos, vec4 color

  :Uniforms:        
    none

2D_SMOOTH_COLOR
  :Attributes:      
    vec3 pos, vec4 color

  :Uniforms:        
    none

2D_IMAGE
  :Attributes:      
    vec3 pos, vec2 texCoord

  :Uniforms:        
    sampler2D image

3D_UNIFORM_COLOR
  :Attributes:      
    vec3 pos

  :Uniforms:        
    vec4 color

3D_FLAT_COLOR
  :Attributes:      
    vec3 pos, vec4 color

  :Uniforms:        
    none

3D_SMOOTH_COLOR
  :Attributes:      
    vec3 pos, vec4 color

  :Uniforms:        
    none

:func:`code_from_builtin`

:func:`from_builtin`

:func:`unbind`

"""

import typing

import bpy

def code_from_builtin(shader_name: str) -> typing.Dict[str, typing.Any]:

  """

  Exposes the internal shader code for query.

  """

  ...

def from_builtin(shader_name: str) -> bpy.types.GPUShader:

  """

  Shaders that are embedded in the blender internal code.
They all read the uniform ``mat4 ModelViewProjectionMatrix``,
which can be edited by the :mod:`gpu.matrix` module.
For more details, you can check the shader code with the
:func:`gpu.shader.code_from_builtin` function.

  """

  ...

def unbind() -> None:

  """

  Unbind the bound shader object.

  """

  ...



"""


bpy_extras submodule (bpy_extras.anim_utils)
********************************************

:func:`bake_action`

:func:`bake_action_objects`

:func:`bake_action_iter`

:func:`bake_action_objects_iter`

"""

import typing

import bpy

def bake_action(obj: bpy.types.Object) -> bpy.types.Action:

  ...

def bake_action_objects(object_action_pairs: typing.Any) -> typing.Sequence[bpy.types.Action]:

  """

  A version of :func:`bake_action_objects_iter` that takes frames and returns the output.

  """

  ...

def bake_action_iter(obj: bpy.types.Object) -> bpy.types.Action:

  """

  An coroutine that bakes action for a single object.

  """

  ...

def bake_action_objects_iter(object_action_pairs: typing.Sequence[typing.Any]) -> None:

  """

  An coroutine that bakes actions for multiple objects.

  """

  ...



"""


Path Utilities (bpy.path)
*************************

This module has a similar scope to os.path, containing utility
functions for dealing with paths in Blender.

:func:`abspath`

:func:`basename`

:func:`clean_name`

:func:`display_name`

:func:`display_name_to_filepath`

:func:`display_name_from_filepath`

:func:`ensure_ext`

:func:`is_subdir`

:func:`module_names`

:func:`native_pathsep`

:func:`reduce_dirs`

:func:`relpath`

:func:`resolve_ncase`

"""

import typing

import bpy

def abspath(path: typing.Any, start: str = None, library: bpy.types.Library = None) -> None:

  """

  Returns the absolute path relative to the current blend file
using the "//" prefix.

  """

  ...

def basename(path: typing.Any) -> None:

  """

  Equivalent to os.path.basename, but skips a "//" prefix.

  Use for Windows compatibility.

  """

  ...

def clean_name(name: typing.Any, replace: typing.Any = '_') -> None:

  """

  Returns a name with characters replaced that
may cause problems under various circumstances,
such as writing to a file.
All characters besides A-Z/a-z, 0-9 are replaced with "_"
or the *replace* argument if defined.

  """

  ...

def display_name(name: typing.Any) -> None:

  """

  Creates a display string from name to be used menus and the user interface.
Capitalize the first letter in all lowercase names,
mixed case names are kept as is. Intended for use with
filenames and module names.

  """

  ...

def display_name_to_filepath(name: typing.Any) -> None:

  """

  Performs the reverse of display_name using literal versions of characters
which aren't supported in a filepath.

  """

  ...

def display_name_from_filepath(name: typing.Any) -> None:

  """

  Returns the path stripped of directory and extension,
ensured to be utf8 compatible.

  """

  ...

def ensure_ext(filepath: typing.Any, ext: str, case_sensitive: bool = False) -> None:

  """

  Return the path with the extension added if it is not already set.

  """

  ...

def is_subdir(path: str, directory: typing.Any) -> None:

  """

  Returns true if *path* in a subdirectory of *directory*.
Both paths must be absolute.

  """

  ...

def module_names(path: str, recursive: bool = False) -> typing.List[typing.Any]:

  """

  Return a list of modules which can be imported from *path*.

  """

  ...

def native_pathsep(path: typing.Any) -> None:

  """

  Replace the path separator with the systems native ``os.sep``.

  """

  ...

def reduce_dirs(dirs: typing.Sequence[typing.Any]) -> typing.List[typing.Any]:

  """

  Given a sequence of directories, remove duplicates and
any directories nested in one of the other paths.
(Useful for recursive path searching).

  """

  ...

def relpath(path: str, start: str = None) -> None:

  """

  Returns the path relative to the current blend file using the "//" prefix.

  """

  ...

def resolve_ncase(path: typing.Any) -> None:

  """

  Resolve a case insensitive path on a case sensitive system,
returning a string with the path if found else return the original path.

  """

  ...



"""


BMesh Types (bmesh.types)
*************************


Base Mesh Type
==============

:class:`BMesh`


Mesh Elements
=============

:class:`BMVert`

:class:`BMEdge`

:class:`BMFace`

:class:`BMLoop`


Sequence Accessors
==================

:class:`BMElemSeq`

:class:`BMVertSeq`

:class:`BMEdgeSeq`

:class:`BMFaceSeq`

:class:`BMLoopSeq`

:class:`BMIter`


Selection History
=================

:class:`BMEditSelSeq`

:class:`BMEditSelIter`


Custom-Data Layer Access
========================

:class:`BMLayerAccessVert`

:class:`BMLayerAccessEdge`

:class:`BMLayerAccessFace`

:class:`BMLayerAccessLoop`

:class:`BMLayerCollection`

:class:`BMLayerItem`


Custom-Data Layer Types
=======================

:class:`BMLoopUV`

:class:`BMDeformVert`

"""

import typing

import mathutils

class BMesh:

  """

  The BMesh data structure

  """

  def calc_loop_triangles(self) -> typing.List[typing.Tuple[BMLoop, ...]]:

    """

    Calculate triangle tessellation from quads/ngons.

    """

    ...

  def calc_volume(self, signed: bool = False) -> float:

    """

    Calculate mesh volume based on face normals.

    """

    ...

  def clear(self) -> None:

    """

    Clear all mesh data.

    """

    ...

  def copy(self) -> BMesh:

    ...

  def free(self) -> None:

    """

    Explicitly free the BMesh data from memory, causing exceptions on further access.

    Note: The BMesh is freed automatically, typically when the script finishes executing.
However in some cases its hard to predict when this will be and its useful to
explicitly free the data.

    """

    ...

  def from_mesh(self, mesh: Mesh, face_normals: typing.Any = True, use_shape_key: bool = False, shape_key_index: int = 0) -> None:

    """

    Initialize this bmesh from existing mesh datablock.

    Note: Multiple calls can be used to join multiple meshes.Custom-data layers are only copied from ``mesh`` on initialization.
Further calls will copy custom-data to matching layers, layers missing on the target mesh wont be added.

    """

    ...

  def from_object(self, object: Object, depsgraph: typing.Any, deform: bool = True, cage: bool = False, face_normals: bool = True) -> None:

    """

    Initialize this bmesh from existing object datablock (currently only meshes are supported).

    """

    ...

  def normal_update(self) -> None:

    """

    Update mesh normals.

    """

    ...

  def select_flush(self, select: bool) -> None:

    """

    Flush selection, independent of the current selection mode.

    """

    ...

  def select_flush_mode(self) -> None:

    """

    flush selection based on the current mode current :class:`BMesh.select_mode`.

    """

    ...

  def to_mesh(self, mesh: Mesh) -> None:

    """

    Writes this BMesh data into an existing Mesh datablock.

    """

    ...

  def transform(self, matrix: typing.Any, filter: typing.Set[typing.Any] = None) -> None:

    """

    Transform the mesh (optionally filtering flagged data only).

    """

    ...

  edges: BMEdgeSeq = ...

  """

  This meshes edge sequence (read-only).

  """

  faces: BMFaceSeq = ...

  """

  This meshes face sequence (read-only).

  """

  is_valid: bool = ...

  """

  True when this element is valid (hasn't been removed).

  """

  is_wrapped: bool = ...

  """

  True when this mesh is owned by blender (typically the editmode BMesh).

  """

  loops: BMLoopSeq = ...

  """

  This meshes loops (read-only).

  Note: Loops must be accessed via faces, this is only exposed for layer access.

  """

  select_history: BMEditSelSeq = ...

  """

  Sequence of selected items (the last is displayed as active).

  """

  select_mode: typing.Set[typing.Any] = ...

  """

  The selection mode, values can be {'VERT', 'EDGE', 'FACE'}, can't be assigned an empty set.

  """

  verts: BMVertSeq = ...

  """

  This meshes vert sequence (read-only).

  """

class BMVert:

  """

  The BMesh vertex type

  """

  def calc_edge_angle(self, fallback: typing.Any = None) -> float:

    """

    Return the angle between this vert's two connected edges.

    """

    ...

  def calc_shell_factor(self) -> float:

    """

    Return a multiplier calculated based on the sharpness of the vertex.
Where a flat surface gives 1.0, and higher values sharper edges.
This is used to maintain shell thickness when offsetting verts along their normals.

    """

    ...

  def copy_from(self, other: typing.Any) -> None:

    """

    Copy values from another element of matching type.

    """

    ...

  def copy_from_face_interp(self, face: BMFace) -> None:

    """

    Interpolate the customdata from a face onto this loop (the loops vert should overlap the face).

    """

    ...

  def copy_from_vert_interp(self, vert_pair: BMVert, fac: typing.Any) -> None:

    """

    Interpolate the customdata from a vert between 2 other verts.

    """

    ...

  def hide_set(self, hide: bool) -> None:

    """

    Set the hide state.
This is different from the *hide* attribute because it updates the selection and hide state of associated geometry.

    """

    ...

  def normal_update(self) -> None:

    """

    Update vertex normal.

    """

    ...

  def select_set(self, select: bool) -> None:

    """

    Set the selection.
This is different from the *select* attribute because it updates the selection state of associated geometry.

    Note: Currently this only flushes down, so selecting a face will select all its vertices but de-selecting a vertex       won't de-select all the faces that use it, before finishing with a mesh typically flushing is still needed.

    """

    ...

  co: mathutils.Vector = ...

  """

  The coordinates for this vertex as a 3D, wrapped vector.

  """

  hide: bool = ...

  """

  Hidden state of this element.

  """

  index: int = ...

  """

  Index of this element.

  Note: This value is not necessarily valid, while editing the mesh it can become *dirty*.It's also possible to assign any number to this attribute for a scripts internal logic.To ensure the value is up to date - see :class:`BMElemSeq.index_update`.

  """

  is_boundary: bool = ...

  """

  True when this vertex is connected to boundary edges (read-only).

  """

  is_manifold: bool = ...

  """

  True when this vertex is manifold (read-only).

  """

  is_valid: bool = ...

  """

  True when this element is valid (hasn't been removed).

  """

  is_wire: bool = ...

  """

  True when this vertex is not connected to any faces (read-only).

  """

  link_edges: BMElemSeq = ...

  """

  Edges connected to this vertex (read-only).

  """

  link_faces: BMElemSeq = ...

  """

  Faces connected to this vertex (read-only).

  """

  link_loops: BMElemSeq = ...

  """

  Loops that use this vertex (read-only).

  """

  normal: mathutils.Vector = ...

  """

  The normal for this vertex as a 3D, wrapped vector.

  """

  select: bool = ...

  """

  Selected state of this element.

  """

  tag: bool = ...

  """

  Generic attribute scripts can use for own logic

  """

class BMEdge:

  """

  The BMesh edge connecting 2 verts

  """

  def calc_face_angle(self, fallback: typing.Any = None) -> float:

    ...

  def calc_face_angle_signed(self, fallback: typing.Any = None) -> float:

    ...

  def calc_length(self) -> float:

    ...

  def calc_tangent(self, loop: BMLoop) -> mathutils.Vector:

    """

    Return the tangent at this edge relative to a face (pointing inward into the face).
This uses the face normal for calculation.

    """

    ...

  def copy_from(self, other: typing.Any) -> None:

    """

    Copy values from another element of matching type.

    """

    ...

  def hide_set(self, hide: bool) -> None:

    """

    Set the hide state.
This is different from the *hide* attribute because it updates the selection and hide state of associated geometry.

    """

    ...

  def normal_update(self) -> None:

    """

    Update edges vertex normals.

    """

    ...

  def other_vert(self, vert: BMVert) -> BMVert:

    """

    Return the other vertex on this edge or None if the vertex is not used by this edge.

    """

    ...

  def select_set(self, select: bool) -> None:

    """

    Set the selection.
This is different from the *select* attribute because it updates the selection state of associated geometry.

    Note: Currently this only flushes down, so selecting a face will select all its vertices but de-selecting a vertex       won't de-select all the faces that use it, before finishing with a mesh typically flushing is still needed.

    """

    ...

  hide: bool = ...

  """

  Hidden state of this element.

  """

  index: int = ...

  """

  Index of this element.

  Note: This value is not necessarily valid, while editing the mesh it can become *dirty*.It's also possible to assign any number to this attribute for a scripts internal logic.To ensure the value is up to date - see :class:`BMElemSeq.index_update`.

  """

  is_boundary: bool = ...

  """

  True when this edge is at the boundary of a face (read-only).

  """

  is_contiguous: bool = ...

  """

  True when this edge is manifold, between two faces with the same winding (read-only).

  """

  is_convex: bool = ...

  """

  True when this edge joins two convex faces, depends on a valid face normal (read-only).

  """

  is_manifold: bool = ...

  """

  True when this edge is manifold (read-only).

  """

  is_valid: bool = ...

  """

  True when this element is valid (hasn't been removed).

  """

  is_wire: bool = ...

  """

  True when this edge is not connected to any faces (read-only).

  """

  link_faces: BMElemSeq = ...

  """

  Faces connected to this edge, (read-only).

  """

  link_loops: BMElemSeq = ...

  """

  Loops connected to this edge, (read-only).

  """

  seam: bool = ...

  """

  Seam for UV unwrapping.

  """

  select: bool = ...

  """

  Selected state of this element.

  """

  smooth: bool = ...

  """

  Smooth state of this element.

  """

  tag: bool = ...

  """

  Generic attribute scripts can use for own logic

  """

  verts: BMElemSeq = ...

  """

  Verts this edge uses (always 2), (read-only).

  """

class BMFace:

  """

  The BMesh face with 3 or more sides

  """

  def calc_area(self) -> float:

    """

    Return the area of the face.

    """

    ...

  def calc_center_bounds(self) -> mathutils.Vector:

    """

    Return bounds center of the face.

    """

    ...

  def calc_center_median(self) -> mathutils.Vector:

    """

    Return median center of the face.

    """

    ...

  def calc_center_median_weighted(self) -> mathutils.Vector:

    """

    Return median center of the face weighted by edge lengths.

    """

    ...

  def calc_perimeter(self) -> float:

    """

    Return the perimeter of the face.

    """

    ...

  def calc_tangent_edge(self) -> mathutils.Vector:

    """

    Return face tangent based on longest edge.

    """

    ...

  def calc_tangent_edge_diagonal(self) -> mathutils.Vector:

    """

    Return face tangent based on the edge farthest from any vertex.

    """

    ...

  def calc_tangent_edge_pair(self) -> mathutils.Vector:

    """

    Return face tangent based on the two longest disconnected edges.

    * Tris: Use the edge pair with the most similar lengths.

    * Quads: Use the longest edge pair.

    * NGons: Use the two longest disconnected edges.

    """

    ...

  def calc_tangent_vert_diagonal(self) -> mathutils.Vector:

    """

    Return face tangent based on the two most distent vertices.

    """

    ...

  def copy(self, verts: bool = True, edges: bool = True) -> BMFace:

    """

    Make a copy of this face.

    """

    ...

  def copy_from(self, other: typing.Any) -> None:

    """

    Copy values from another element of matching type.

    """

    ...

  def copy_from_face_interp(self, face: BMFace, vert: bool = True) -> None:

    """

    Interpolate the customdata from another face onto this one (faces should overlap).

    """

    ...

  def hide_set(self, hide: bool) -> None:

    """

    Set the hide state.
This is different from the *hide* attribute because it updates the selection and hide state of associated geometry.

    """

    ...

  def normal_flip(self) -> None:

    """

    Reverses winding of a face, which flips its normal.

    """

    ...

  def normal_update(self) -> None:

    """

    Update face's normal.

    """

    ...

  def select_set(self, select: bool) -> None:

    """

    Set the selection.
This is different from the *select* attribute because it updates the selection state of associated geometry.

    Note: Currently this only flushes down, so selecting a face will select all its vertices but de-selecting a vertex       won't de-select all the faces that use it, before finishing with a mesh typically flushing is still needed.

    """

    ...

  edges: BMElemSeq = ...

  """

  Edges of this face, (read-only).

  """

  hide: bool = ...

  """

  Hidden state of this element.

  """

  index: int = ...

  """

  Index of this element.

  Note: This value is not necessarily valid, while editing the mesh it can become *dirty*.It's also possible to assign any number to this attribute for a scripts internal logic.To ensure the value is up to date - see :class:`BMElemSeq.index_update`.

  """

  is_valid: bool = ...

  """

  True when this element is valid (hasn't been removed).

  """

  loops: BMElemSeq = ...

  """

  Loops of this face, (read-only).

  """

  material_index: int = ...

  """

  The face's material index.

  """

  normal: mathutils.Vector = ...

  """

  The normal for this face as a 3D, wrapped vector.

  """

  select: bool = ...

  """

  Selected state of this element.

  """

  smooth: bool = ...

  """

  Smooth state of this element.

  """

  tag: bool = ...

  """

  Generic attribute scripts can use for own logic

  """

  verts: BMElemSeq = ...

  """

  Verts of this face, (read-only).

  """

class BMLoop:

  """

  This is normally accessed from :class:`BMFace.loops` where each face loop represents a corner of the face.

  """

  def calc_angle(self) -> float:

    """

    Return the angle at this loops corner of the face.
This is calculated so sharper corners give lower angles.

    """

    ...

  def calc_normal(self) -> mathutils.Vector:

    """

    Return normal at this loops corner of the face.
Falls back to the face normal for straight lines.

    """

    ...

  def calc_tangent(self) -> mathutils.Vector:

    """

    Return the tangent at this loops corner of the face (pointing inward into the face).
Falls back to the face normal for straight lines.

    """

    ...

  def copy_from(self, other: typing.Any) -> None:

    """

    Copy values from another element of matching type.

    """

    ...

  def copy_from_face_interp(self, face: BMFace, vert: bool = True, multires: bool = True) -> None:

    """

    Interpolate the customdata from a face onto this loop (the loops vert should overlap the face).

    """

    ...

  edge: BMEdge = ...

  """

  The loop's edge (between this loop and the next), (read-only).

  """

  face: BMFace = ...

  """

  The face this loop makes (read-only).

  """

  index: int = ...

  """

  Index of this element.

  Note: This value is not necessarily valid, while editing the mesh it can become *dirty*.It's also possible to assign any number to this attribute for a scripts internal logic.To ensure the value is up to date - see :class:`BMElemSeq.index_update`.

  """

  is_convex: bool = ...

  """

  True when this loop is at the convex corner of a face, depends on a valid face normal (read-only).

  """

  is_valid: bool = ...

  """

  True when this element is valid (hasn't been removed).

  """

  link_loop_next: BMLoop = ...

  """

  The next face corner (read-only).

  """

  link_loop_prev: BMLoop = ...

  """

  The previous face corner (read-only).

  """

  link_loop_radial_next: BMLoop = ...

  """

  The next loop around the edge (read-only).

  """

  link_loop_radial_prev: BMLoop = ...

  """

  The previous loop around the edge (read-only).

  """

  link_loops: BMElemSeq = ...

  """

  Loops connected to this loop, (read-only).

  """

  tag: bool = ...

  """

  Generic attribute scripts can use for own logic

  """

  vert: BMVert = ...

  """

  The loop's vertex (read-only).

  """

class BMElemSeq:

  """

  General sequence type used for accessing any sequence of
:class:`BMVert`, :class:`BMEdge`, :class:`BMFace`, :class:`BMLoop`.

  When accessed via :class:`BMesh.verts`, :class:`BMesh.edges`, :class:`BMesh.faces`
there are also functions to create/remomove items.

  """

  def index_update(self) -> None:

    """

    Initialize the index values of this sequence.

    This is the equivalent of looping over all elements and assigning the index values.

    .. code:: python

      for index, ele in enumerate(sequence):
          ele.index = index

    Note: Running this on sequences besides :class:`BMesh.verts`, :class:`BMesh.edges`, :class:`BMesh.faces`
works but wont result in each element having a valid index, instead its order in the sequence will be set.

    """

    ...

class BMVertSeq:

  def ensure_lookup_table(self) -> None:

    """

    Ensure internal data needed for int subscription is initialized with verts/edges/faces, eg ``bm.verts[index]``.

    This needs to be called again after adding/removing data in this sequence.

    """

    ...

  def index_update(self) -> None:

    """

    Initialize the index values of this sequence.

    This is the equivalent of looping over all elements and assigning the index values.

    .. code:: python

      for index, ele in enumerate(sequence):
          ele.index = index

    Note: Running this on sequences besides :class:`BMesh.verts`, :class:`BMesh.edges`, :class:`BMesh.faces`
works but wont result in each element having a valid index, instead its order in the sequence will be set.

    """

    ...

  def new(self, co: float = (0.0, 0.0, 0.0), example: BMVert = None) -> BMVert:

    """

    Create a new vertex.

    """

    ...

  def remove(self, vert: typing.Any) -> None:

    """

    Remove a vert.

    """

    ...

  def sort(self, key: typing.Callable = None, reverse: bool = False) -> None:

    """

    Sort the elements of this sequence, using an optional custom sort key.
Indices of elements are not changed, BMElemeSeq.index_update() can be used for that.

    Note: When the 'key' argument is not provided, the elements are reordered following their current index value.
In particular this can be used by setting indices manually before calling this method.

    Warning: Existing references to the N'th element, will continue to point the data at that index.

    """

    ...

  layers: BMLayerAccessVert = ...

  """

  custom-data layers (read-only).

  """

class BMEdgeSeq:

  """"""

  def ensure_lookup_table(self) -> None:

    """

    Ensure internal data needed for int subscription is initialized with verts/edges/faces, eg ``bm.verts[index]``.

    This needs to be called again after adding/removing data in this sequence.

    """

    ...

  def get(self, verts: BMVert, fallback: typing.Any = None) -> BMEdge:

    """

    Return an edge which uses the **verts** passed.

    """

    ...

  def index_update(self) -> None:

    """

    Initialize the index values of this sequence.

    This is the equivalent of looping over all elements and assigning the index values.

    .. code:: python

      for index, ele in enumerate(sequence):
          ele.index = index

    Note: Running this on sequences besides :class:`BMesh.verts`, :class:`BMesh.edges`, :class:`BMesh.faces`
works but wont result in each element having a valid index, instead its order in the sequence will be set.

    """

    ...

  def new(self, verts: typing.Any, example: BMEdge = None) -> BMEdge:

    """

    Create a new edge from a given pair of verts.

    """

    ...

  def remove(self, edge: typing.Any) -> None:

    """

    Remove an edge.

    """

    ...

  def sort(self, key: typing.Callable = None, reverse: bool = False) -> None:

    """

    Sort the elements of this sequence, using an optional custom sort key.
Indices of elements are not changed, BMElemeSeq.index_update() can be used for that.

    Note: When the 'key' argument is not provided, the elements are reordered following their current index value.
In particular this can be used by setting indices manually before calling this method.

    Warning: Existing references to the N'th element, will continue to point the data at that index.

    """

    ...

  layers: BMLayerAccessEdge = ...

  """

  custom-data layers (read-only).

  """

class BMFaceSeq:

  def ensure_lookup_table(self) -> None:

    """

    Ensure internal data needed for int subscription is initialized with verts/edges/faces, eg ``bm.verts[index]``.

    This needs to be called again after adding/removing data in this sequence.

    """

    ...

  def get(self, verts: BMVert, fallback: typing.Any = None) -> BMFace:

    """

    Return a face which uses the **verts** passed.

    """

    ...

  def index_update(self) -> None:

    """

    Initialize the index values of this sequence.

    This is the equivalent of looping over all elements and assigning the index values.

    .. code:: python

      for index, ele in enumerate(sequence):
          ele.index = index

    Note: Running this on sequences besides :class:`BMesh.verts`, :class:`BMesh.edges`, :class:`BMesh.faces`
works but wont result in each element having a valid index, instead its order in the sequence will be set.

    """

    ...

  def new(self, verts: BMVert, example: BMFace = None) -> BMFace:

    """

    Create a new face from a given set of verts.

    """

    ...

  def remove(self, face: typing.Any) -> None:

    """

    Remove a face.

    """

    ...

  def sort(self, key: typing.Callable = None, reverse: bool = False) -> None:

    """

    Sort the elements of this sequence, using an optional custom sort key.
Indices of elements are not changed, BMElemeSeq.index_update() can be used for that.

    Note: When the 'key' argument is not provided, the elements are reordered following their current index value.
In particular this can be used by setting indices manually before calling this method.

    Warning: Existing references to the N'th element, will continue to point the data at that index.

    """

    ...

  active: BMFace = ...

  """

  active face.

  """

  layers: BMLayerAccessFace = ...

  """

  custom-data layers (read-only).

  """

class BMLoopSeq:

  layers: BMLayerAccessLoop = ...

  """

  custom-data layers (read-only).

  """

class BMIter:

  """

  Internal BMesh type for looping over verts/faces/edges,
used for iterating over :class:`BMElemSeq` types.

  """

  ...

class BMEditSelSeq:

  def add(self, element: typing.Any) -> None:

    """

    Add an element to the selection history (no action taken if its already added).

    """

    ...

  def clear(self) -> None:

    """

    Empties the selection history.

    """

    ...

  def discard(self, element: typing.Any) -> None:

    """

    Discard an element from the selection history.

    Like remove but doesn't raise an error when the elements not in the selection list.

    """

    ...

  def remove(self, element: typing.Any) -> None:

    """

    Remove an element from the selection history.

    """

    ...

  def validate(self) -> None:

    """

    Ensures all elements in the selection history are selected.

    """

    ...

  active: BMVert = ...

  """

  The last selected element or None (read-only).

  """

class BMEditSelIter:

  ...

class BMLayerAccessVert:

  """

  Exposes custom-data layer attributes.

  """

  bevel_weight: BMLayerCollection = ...

  """

  Bevel weight float in [0 - 1].

  """

  color: typing.Any = ...

  """

  Generic RGBA color with 8-bit precision custom-data layer.

  type: :class:`BMLayerCollection`

  """

  deform: typing.Any = ...

  """

  Vertex deform weight :class:`BMDeformVert` (TODO).

  type: :class:`BMLayerCollection`

  """

  float: typing.Any = ...

  """

  Generic float custom-data layer.

  type: :class:`BMLayerCollection`

  """

  float_color: typing.Any = ...

  """

  Generic RGBA color with float precision custom-data layer.

  type: :class:`BMLayerCollection`

  """

  float_vector: typing.Any = ...

  """

  Generic 3D vector with float precision custom-data layer.

  type: :class:`BMLayerCollection`

  """

  int: typing.Any = ...

  """

  Generic int custom-data layer.

  type: :class:`BMLayerCollection`

  """

  paint_mask: typing.Any = ...

  """

  Accessor for paint mask layer.

  type: :class:`BMLayerCollection`

  """

  shape: BMLayerCollection = ...

  """

  Vertex shapekey absolute location (as a 3D Vector).

  """

  skin: typing.Any = ...

  """

  Accessor for skin layer.

  type: :class:`BMLayerCollection`

  """

  string: typing.Any = ...

  """

  Generic string custom-data layer (exposed as bytes, 255 max length).

  type: :class:`BMLayerCollection`

  """

class BMLayerAccessEdge:

  """

  Exposes custom-data layer attributes.

  """

  bevel_weight: BMLayerCollection = ...

  """

  Bevel weight float in [0 - 1].

  """

  color: typing.Any = ...

  """

  Generic RGBA color with 8-bit precision custom-data layer.

  type: :class:`BMLayerCollection`

  """

  crease: BMLayerCollection = ...

  """

  Edge crease for subdivision surface - float in [0 - 1].

  """

  float: typing.Any = ...

  """

  Generic float custom-data layer.

  type: :class:`BMLayerCollection`

  """

  float_color: typing.Any = ...

  """

  Generic RGBA color with float precision custom-data layer.

  type: :class:`BMLayerCollection`

  """

  float_vector: typing.Any = ...

  """

  Generic 3D vector with float precision custom-data layer.

  type: :class:`BMLayerCollection`

  """

  freestyle: typing.Any = ...

  """

  Accessor for Freestyle edge layer.

  type: :class:`BMLayerCollection`

  """

  int: typing.Any = ...

  """

  Generic int custom-data layer.

  type: :class:`BMLayerCollection`

  """

  string: typing.Any = ...

  """

  Generic string custom-data layer (exposed as bytes, 255 max length).

  type: :class:`BMLayerCollection`

  """

class BMLayerAccessFace:

  """

  Exposes custom-data layer attributes.

  """

  color: typing.Any = ...

  """

  Generic RGBA color with 8-bit precision custom-data layer.

  type: :class:`BMLayerCollection`

  """

  face_map: typing.Any = ...

  """

  FaceMap custom-data layer.

  type: :class:`BMLayerCollection`

  """

  float: typing.Any = ...

  """

  Generic float custom-data layer.

  type: :class:`BMLayerCollection`

  """

  float_color: typing.Any = ...

  """

  Generic RGBA color with float precision custom-data layer.

  type: :class:`BMLayerCollection`

  """

  float_vector: typing.Any = ...

  """

  Generic 3D vector with float precision custom-data layer.

  type: :class:`BMLayerCollection`

  """

  freestyle: typing.Any = ...

  """

  Accessor for Freestyle face layer.

  type: :class:`BMLayerCollection`

  """

  int: typing.Any = ...

  """

  Generic int custom-data layer.

  type: :class:`BMLayerCollection`

  """

  string: typing.Any = ...

  """

  Generic string custom-data layer (exposed as bytes, 255 max length).

  type: :class:`BMLayerCollection`

  """

class BMLayerAccessLoop:

  """

  Exposes custom-data layer attributes.

  """

  color: typing.Any = ...

  """

  Generic RGBA color with 8-bit precision custom-data layer.

  type: :class:`BMLayerCollection`

  """

  float: typing.Any = ...

  """

  Generic float custom-data layer.

  type: :class:`BMLayerCollection`

  """

  float_color: typing.Any = ...

  """

  Generic RGBA color with float precision custom-data layer.

  type: :class:`BMLayerCollection`

  """

  float_vector: typing.Any = ...

  """

  Generic 3D vector with float precision custom-data layer.

  type: :class:`BMLayerCollection`

  """

  int: typing.Any = ...

  """

  Generic int custom-data layer.

  type: :class:`BMLayerCollection`

  """

  string: typing.Any = ...

  """

  Generic string custom-data layer (exposed as bytes, 255 max length).

  type: :class:`BMLayerCollection`

  """

  uv: typing.Any = ...

  """

  Accessor for :class:`BMLoopUV` UV (as a 2D Vector).

  type: :class:`BMLayerCollection`

  """

class BMLayerCollection:

  """

  Gives access to a collection of custom-data layers of the same type and behaves like python dictionaries, except for the ability to do list like index access.

  """

  def get(self, key: str, default: typing.Any = None) -> None:

    """

    Returns the value of the layer matching the key or default
when not found (matches pythons dictionary function of the same name).

    """

    ...

  def items(self) -> typing.List[typing.Tuple[typing.Any, ...]]:

    """

    Return the identifiers of collection members
(matching pythons dict.items() functionality).

    """

    ...

  def keys(self) -> typing.List[str]:

    """

    Return the identifiers of collection members
(matching pythons dict.keys() functionality).

    """

    ...

  def new(self, name: str) -> BMLayerItem:

    """

    Create a new layer

    """

    ...

  def remove(self, layer: BMLayerItem) -> None:

    """

    Remove a layer

    """

    ...

  def values(self) -> typing.List[typing.Any]:

    """

    Return the values of collection
(matching pythons dict.values() functionality).

    """

    ...

  def verify(self) -> BMLayerItem:

    """

    Create a new layer or return an existing active layer

    """

    ...

  active: BMLayerItem = ...

  """

  The active layer of this type (read-only).

  """

  is_singleton: bool = ...

  """

  True if there can exists only one layer of this type (read-only).

  """

class BMLayerItem:

  """

  Exposes a single custom data layer, their main purpose is for use as item accessors to custom-data when used with vert/edge/face/loop data.

  """

  def copy_from(self, other: typing.Any) -> None:

    """

    Return a copy of the layer

    """

    ...

  name: str = ...

  """

  The layers unique name (read-only).

  """

class BMLoopUV:

  pin_uv: bool = ...

  """

  UV pin state.

  """

  select: bool = ...

  """

  UV select state.

  """

  uv: mathutils.Vector = ...

  """

  Loops UV (as a 2D Vector).

  """

class BMDeformVert:

  """"""

  def clear(self) -> None:

    """

    Clears all weights.

    """

    ...

  def get(self, key: int, default: typing.Any = None) -> None:

    """

    Returns the deform weight matching the key or default
when not found (matches pythons dictionary function of the same name).

    """

    ...

  def items(self) -> typing.List[typing.Tuple[typing.Any, ...]]:

    """

    Return (group, weight) pairs for this vertex
(matching pythons dict.items() functionality).

    """

    ...

  def keys(self) -> typing.List[int]:

    """

    Return the group indices used by this vertex
(matching pythons dict.keys() functionality).

    """

    ...

  def values(self) -> typing.List[float]:

    """

    Return the weights of the deform vertex
(matching pythons dict.values() functionality).

    """

    ...



"""


Export Scene Operators
**********************

:func:`fbx`

:func:`gltf`

:func:`obj`

:func:`x3d`

"""

import typing

def fbx(filepath: str = '', check_existing: bool = True, filter_glob: str = '*.fbx', use_selection: bool = False, use_active_collection: bool = False, global_scale: float = 1.0, apply_unit_scale: bool = True, apply_scale_options: str = 'FBX_SCALE_NONE', bake_space_transform: bool = False, object_types: typing.Set[str] = {'ARMATURE', 'CAMERA', 'EMPTY', 'LIGHT', 'MESH', 'OTHER'}, use_mesh_modifiers: bool = True, use_mesh_modifiers_render: bool = True, mesh_smooth_type: str = 'OFF', use_subsurf: bool = False, use_mesh_edges: bool = False, use_tspace: bool = False, use_custom_props: bool = False, add_leaf_bones: bool = True, primary_bone_axis: str = 'Y', secondary_bone_axis: str = 'X', use_armature_deform_only: bool = False, armature_nodetype: str = 'NULL', bake_anim: bool = True, bake_anim_use_all_bones: bool = True, bake_anim_use_nla_strips: bool = True, bake_anim_use_all_actions: bool = True, bake_anim_force_startend_keying: bool = True, bake_anim_step: float = 1.0, bake_anim_simplify_factor: float = 1.0, path_mode: str = 'AUTO', embed_textures: bool = False, batch_mode: str = 'OFF', use_batch_own_dir: bool = True, use_metadata: bool = True, axis_forward: str = '-Z', axis_up: str = 'Y') -> None:

  """

  Write a FBX file

  """

  ...

def gltf(export_format: str = 'GLB', ui_tab: str = 'GENERAL', export_copyright: str = '', export_image_format: str = 'AUTO', export_texture_dir: str = '', export_texcoords: bool = True, export_normals: bool = True, export_draco_mesh_compression_enable: bool = False, export_draco_mesh_compression_level: int = 6, export_draco_position_quantization: int = 14, export_draco_normal_quantization: int = 10, export_draco_texcoord_quantization: int = 12, export_draco_generic_quantization: int = 12, export_tangents: bool = False, export_materials: str = 'EXPORT', export_colors: bool = True, export_cameras: bool = False, export_selected: bool = False, use_selection: bool = False, export_extras: bool = False, export_yup: bool = True, export_apply: bool = False, export_animations: bool = True, export_frame_range: bool = True, export_frame_step: int = 1, export_force_sampling: bool = True, export_nla_strips: bool = True, export_def_bones: bool = False, export_current_frame: bool = False, export_skins: bool = True, export_all_influences: bool = False, export_morph: bool = True, export_morph_normal: bool = True, export_morph_tangent: bool = False, export_lights: bool = False, export_displacement: bool = False, will_save_settings: bool = False, filepath: str = '', check_existing: bool = True, filter_glob: str = '*.glb;*.gltf') -> None:

  """

  Export scene as glTF 2.0 file

  """

  ...

def obj(filepath: str = '', check_existing: bool = True, filter_glob: str = '*.obj;*.mtl', use_selection: bool = False, use_animation: bool = False, use_mesh_modifiers: bool = True, use_edges: bool = True, use_smooth_groups: bool = False, use_smooth_groups_bitflags: bool = False, use_normals: bool = True, use_uvs: bool = True, use_materials: bool = True, use_triangles: bool = False, use_nurbs: bool = False, use_vertex_groups: bool = False, use_blen_objects: bool = True, group_by_object: bool = False, group_by_material: bool = False, keep_vertex_order: bool = False, global_scale: float = 1.0, path_mode: str = 'AUTO', axis_forward: str = '-Z', axis_up: str = 'Y') -> None:

  """

  Save a Wavefront OBJ File

  """

  ...

def x3d(filepath: str = '', check_existing: bool = True, filter_glob: str = '*.x3d', use_selection: bool = False, use_mesh_modifiers: bool = True, use_triangulate: bool = False, use_normals: bool = False, use_compress: bool = False, use_hierarchy: bool = True, name_decorations: bool = True, use_h3d: bool = False, global_scale: float = 1.0, path_mode: str = 'AUTO', axis_forward: str = 'Z', axis_up: str = 'Y') -> None:

  """

  Export selection to Extensible 3D file (.x3d)

  """

  ...



"""


Gizmogroup Operators
********************

:func:`gizmo_select`

:func:`gizmo_tweak`

"""

import typing

def gizmo_select(extend: bool = False, deselect: bool = False, toggle: bool = False, deselect_all: bool = False) -> None:

  """

  Select the currently highlighted gizmo

  """

  ...

def gizmo_tweak() -> None:

  """

  Tweak the active gizmo

  """

  ...



"""


Application Translations (bpy.app.translations)
***********************************************

This object contains some data/methods regarding internationalization in Blender, and allows every py script
to feature translations for its own UI messages.


Introduction
============

Warning: Most of this object should only be useful if you actually manipulate i18n stuff from Python.
If you are a regular add-on, you should only bother about :const:`contexts` member,
and the :func:`register`/:func:`unregister` functions! The :func:`pgettext` family of functions
should only be used in rare, specific cases (like e.g. complex "composited" UI strings...).

To add translations to your python script, you must define a dictionary formatted like that:
``{locale: {msg_key: msg_translation, ...}, ...}`` where:

* locale is either a lang iso code (e.g. ``fr``), a lang+country code (e.g. ``pt_BR``),
a lang+variant code (e.g. ``sr@latin``), or a full code (e.g. ``uz_UZ@cyrilic``).

* msg_key is a tuple (context, org message) - use, as much as possible, the predefined :const:`contexts`.

* msg_translation is the translated message in given language!

Then, call ``bpy.app.translations.register(__name__, your_dict)`` in your ``register()`` function, and
``bpy.app.translations.unregister(__name__)`` in your ``unregister()`` one.

The ``Manage UI translations`` add-on has several functions to help you collect strings to translate, and
generate the needed python code (the translation dictionary), as well as optional intermediary po files
if you want some... See
`How to Translate Blender <https://wiki.blender.org/wiki/Process/Translate_Blender>`_ and
`Using i18n in Blender Code <https://wiki.blender.org/wiki/Source/Interface/Internationalization>`_
for more info.


Module References
=================

.. code::


  import bpy

  # This block can be automatically generated by UI translations addon, which also handles conversion with PO format.
  # See also https://wiki.blender.org/wiki/Process/Translate_Blender#Translating_non-official_addons
  # It can (should) also be put in a different, specific py file.

  # ##### BEGIN AUTOGENERATED I18N SECTION #####
  # NOTE: You can safely move around this auto-generated block (with the begin/end markers!),
  #       and edit the translations by hand.
  #       Just carefully respect the format of the tuple!

  # Tuple of tuples ((msgctxt, msgid), (sources, gen_comments), (lang, translation, (is_fuzzy, comments)), ...)
  translations_tuple = (
      (("*", ""),
       ((), ()),
       ("fr_FR", "Project-Id-Version: Copy Settings 0.1.5 (r0)\nReport-Msgid-Bugs-To: \nPOT-Creation-Date: 2013-04-18 15:27:45.563524\nPO-Revision-Date: 2013-04-18 15:38+0100\nLast-Translator: Bastien Montagne <montagne29@wanadoo.fr>\nLanguage-Team: LANGUAGE <LL@li.org>\nLanguage: __POT__\nMIME-Version: 1.0\nContent-Type: text/plain; charset=UTF-8\nContent-Transfer-Encoding: 8bit\n",
                 (False,
                  ("Blender's translation file (po format).",
                   "Copyright (C) 2013 The Blender Foundation.",
                   "This file is distributed under the same license as the Blender package.",
                   "FIRST AUTHOR <EMAIL@ADDRESS>, YEAR."))),
       ),
      (("Operator", "Render: Copy Settings"),
       (("bpy.types.SCENE_OT_render_copy_settings",),
        ()),
       ("fr_FR", "Rendu : copier réglages",
                 (False, ())),
       ),
      (("*", "Copy render settings from current scene to others"),
       (("bpy.types.SCENE_OT_render_copy_settings",),
        ()),
       ("fr_FR", "Copier les réglages de rendu depuis la scène courante vers d’autres",
                 (False, ())),
       ),
      # ... etc, all messages from your addon.
  )

  translations_dict = {}
  for msg in translations_tuple:
      key = msg[0]
      for lang, trans, (is_fuzzy, comments) in msg[2:]:
          if trans and not is_fuzzy:
              translations_dict.setdefault(lang, {})[key] = trans

  # ##### END AUTOGENERATED I18N SECTION #####

  # Define remaining addon (operators, UI...) here.


  def register():
     # Usual operator/UI/etc. registration...

      bpy.app.translations.register(__name__, translations_dict)


  def unregister():
      bpy.app.translations.unregister(__name__)

     # Usual operator/UI/etc. unregistration...

:data:`locale`

:data:`locales`

:data:`contexts_C_to_py`

:data:`contexts`

:func:`locale_explode`

:func:`pgettext`

:func:`pgettext_data`

:func:`pgettext_iface`

:func:`pgettext_tip`

:func:`register`

:func:`unregister`

"""

import typing

locale: typing.Any = ...

"""

The actual locale currently in use (will always return a void string when Blender is built without internationalization support).

"""

locales: typing.Any = ...

"""

All locales currently known by Blender (i.e. available as translations).

"""

contexts_C_to_py: typing.Any = ...

"""

A readonly dict mapping contexts' C-identifiers to their py-identifiers.

"""

contexts: typing.Any = ...

"""

constant value bpy.app.translations.contexts(default_real=None, default='*', operator_default='Operator', ui_events_keymaps='UI_Events_KeyMaps', plural='Plural', id_action='Action', id_armature='Armature', id_brush='Brush', id_camera='Camera', id_cachefile='CacheFile', id_collection='Collection', id_curve='Curve', id_fs_linestyle='FreestyleLineStyle', id_gpencil='GPencil', id_hair='Hair', id_id='ID', id_image='Image', id_shapekey='Key', id_light='Light', id_library='Library', id_lattice='Lattice', id_mask='Mask', ...)

"""

def locale_explode(self, locale: typing.Any) -> None:

  """

  Return all components and their combinations  of the given ISO locale string.

  >>> bpy.app.translations.locale_explode("sr_RS@latin")
  ("sr", "RS", "latin", "sr_RS", "sr@latin")

  For non-complete locales, missing elements will be None.

  """

  ...

def pgettext(self, msgid: str, msgctxt: str = None) -> None:

  """

  Try to translate the given msgid (with optional msgctxt).

  Note: The ``(msgid, msgctxt)`` parameters order has been switched compared to gettext function, to allow
single-parameter calls (context then defaults to BLT_I18NCONTEXT_DEFAULT).

  Note: You should really rarely need to use this function in regular addon code, as all translation should be
handled by Blender internal code. The only exception are string containing formatting (like "File: %r"),
but you should rather use :func:`pgettext_iface`/:func:`pgettext_tip` in those cases!

  Note: Does nothing when Blender is built without internationalization support (hence always returns ``msgid``).

  """

  ...

def pgettext_data(self, msgid: str, msgctxt: str = None) -> None:

  """

  Try to translate the given msgid (with optional msgctxt), if new data name's translation is enabled.

  Note: See :func:`pgettext` notes.

  """

  ...

def pgettext_iface(self, msgid: str, msgctxt: str = None) -> None:

  """

  Try to translate the given msgid (with optional msgctxt), if labels' translation is enabled.

  Note: See :func:`pgettext` notes.

  """

  ...

def pgettext_tip(self, msgid: str, msgctxt: str = None) -> None:

  """

  Try to translate the given msgid (with optional msgctxt), if tooltips' translation is enabled.

  Note: See :func:`pgettext` notes.

  """

  ...

def register(self, module_name: str, translations_dict: typing.Dict[str, typing.Any]) -> None:

  """

  Registers an addon's UI translations.

  Note: Does nothing when Blender is built without internationalization support.

  """

  ...

def unregister(self, module_name: str) -> None:

  """

  Unregisters an addon's UI translations.

  Note: Does nothing when Blender is built without internationalization support.

  """

  ...



"""


Message Bus (bpy.msgbus)
************************

The message bus system can be used to receive notifications when properties of
Blender datablocks are changed via the data API.


Limitations
===========

The message bus system is triggered by updates via the RNA system. This means
that the following updates will result in a notification on the message bus:

* Changes via the Python API, for example ``some_object.location.x += 3``.

* Changes via the sliders, fields, and buttons in the user interface.

The following updates do **not** trigger message bus notifications:

* Moving objects in the 3D Viewport.

* Changes performed by the animation system.


Example Use
===========

Below is an example of subscription to changes in the active object's location.

.. code::


  import bpy

  # Any Python object can act as the subscription's owner.
  owner = object()

  subscribe_to = bpy.context.object.location

  def msgbus_callback(*args):
      # This will print:
      # Something changed! (1, 2, 3)
      print("Something changed!", args)

  bpy.msgbus.subscribe_rna(
      key=subscribe_to,
      owner=owner,
      args=(1, 2, 3),
      notify=msgbus_callback,
  )

Some properties are converted to Python objects when you retrieve them. This
needs to be avoided in order to create the subscription, by using
``datablock.path_resolve("property_name", False)``:

.. code::

  subscribe_to = bpy.context.object.path_resolve("name", False)

It is also possible to create subscriptions on a property of all instances of a
certain type:

.. code::

  subscribe_to = (bpy.types.Object, "location")

:func:`clear_by_owner`

:func:`publish_rna`

:func:`subscribe_rna`

"""

import typing

def clear_by_owner(owner: typing.Any) -> None:

  """

  Clear all subscribers using this owner.

  """

  ...

def publish_rna(key: typing.Any) -> None:

  """

  Notify subscribers of changes to this property
(this typically doesn't need to be called explicitly since changes will automatically publish updates).
In some cases it may be useful to publish changes explicitly using more general keys.

  """

  ...

def subscribe_rna(key: typing.Any, owner: typing.Any, args: typing.Any, notify: typing.Any, options: typing.Set[str] = set()) -> None:

  ...



"""


Import Scene Operators
**********************

:func:`fbx`

:func:`gltf`

:func:`obj`

:func:`x3d`

"""

import typing

def fbx(filepath: str = '', directory: str = '', filter_glob: str = '*.fbx', files: typing.Union[typing.Sequence[OperatorFileListElement], typing.Mapping[str, OperatorFileListElement], bpy.types.bpy_prop_collection] = None, ui_tab: str = 'MAIN', use_manual_orientation: bool = False, global_scale: float = 1.0, bake_space_transform: bool = False, use_custom_normals: bool = True, use_image_search: bool = True, use_alpha_decals: bool = False, decal_offset: float = 0.0, use_anim: bool = True, anim_offset: float = 1.0, use_subsurf: bool = False, use_custom_props: bool = True, use_custom_props_enum_as_string: bool = True, ignore_leaf_bones: bool = False, force_connect_children: bool = False, automatic_bone_orientation: bool = False, primary_bone_axis: str = 'Y', secondary_bone_axis: str = 'X', use_prepost_rot: bool = True, axis_forward: str = '-Z', axis_up: str = 'Y') -> None:

  """

  Load a FBX file

  """

  ...

def gltf(filepath: str = '', filter_glob: str = '*.glb;*.gltf', files: typing.Union[typing.Sequence[OperatorFileListElement], typing.Mapping[str, OperatorFileListElement], bpy.types.bpy_prop_collection] = None, loglevel: int = 0, import_pack_images: bool = True, merge_vertices: bool = False, import_shading: str = 'NORMALS', bone_heuristic: str = 'TEMPERANCE', guess_original_bind_pose: bool = True) -> None:

  """

  Load a glTF 2.0 file

  """

  ...

def obj(filepath: str = '', filter_glob: str = '*.obj;*.mtl', use_edges: bool = True, use_smooth_groups: bool = True, use_split_objects: bool = True, use_split_groups: bool = False, use_groups_as_vgroups: bool = False, use_image_search: bool = True, split_mode: str = 'ON', global_clight_size: float = 0.0, axis_forward: str = '-Z', axis_up: str = 'Y') -> None:

  """

  Load a Wavefront OBJ File

  """

  ...

def x3d(filepath: str = '', filter_glob: str = '*.x3d;*.wrl', axis_forward: str = 'Z', axis_up: str = 'Y') -> None:

  """

  Import an X3D or VRML2 file

  """

  ...



"""


Simulation Operators
********************

:func:`new`

"""

import typing

def new() -> None:

  """

  Create a new simulation data block and edit it in the opened simulation editor

  """

  ...



"""


View2D Operators
****************

:func:`edge_pan`

:func:`ndof`

:func:`pan`

:func:`reset`

:func:`scroll_down`

:func:`scroll_left`

:func:`scroll_right`

:func:`scroll_up`

:func:`scroller_activate`

:func:`smoothview`

:func:`zoom`

:func:`zoom_border`

:func:`zoom_in`

:func:`zoom_out`

"""

import typing

def edge_pan(outside_padding: int = 0) -> None:

  """

  Pan the view when the mouse is held at an edge

  """

  ...

def ndof() -> None:

  """

  Use a 3D mouse device to pan/zoom the view

  """

  ...

def pan(deltax: int = 0, deltay: int = 0) -> None:

  """

  Pan the view

  """

  ...

def reset() -> None:

  """

  Reset the view

  """

  ...

def scroll_down(deltax: int = 0, deltay: int = 0, page: bool = False) -> None:

  """

  Scroll the view down

  """

  ...

def scroll_left(deltax: int = 0, deltay: int = 0) -> None:

  """

  Scroll the view left

  """

  ...

def scroll_right(deltax: int = 0, deltay: int = 0) -> None:

  """

  Scroll the view right

  """

  ...

def scroll_up(deltax: int = 0, deltay: int = 0, page: bool = False) -> None:

  """

  Scroll the view up

  """

  ...

def scroller_activate() -> None:

  """

  Scroll view by mouse click and drag

  """

  ...

def smoothview(xmin: int = 0, xmax: int = 0, ymin: int = 0, ymax: int = 0, wait_for_input: bool = True) -> None:

  """

  Undocumented, consider `contributing <https://developer.blender.org/T51061>`_.

  """

  ...

def zoom(deltax: float = 0.0, deltay: float = 0.0, use_cursor_init: bool = True) -> None:

  """

  Zoom in/out the view

  """

  ...

def zoom_border(xmin: int = 0, xmax: int = 0, ymin: int = 0, ymax: int = 0, wait_for_input: bool = True, zoom_out: bool = False) -> None:

  """

  Zoom in the view to the nearest item contained in the border

  """

  ...

def zoom_in(zoomfacx: float = 0.0, zoomfacy: float = 0.0) -> None:

  """

  Zoom in the view

  """

  ...

def zoom_out(zoomfacx: float = 0.0, zoomfacy: float = 0.0) -> None:

  """

  Zoom out the view

  """

  ...



"""


Pose Operators
**************

:func:`armature_apply`

:func:`autoside_names`

:func:`bone_layers`

:func:`breakdown`

:func:`constraint_add`

:func:`constraint_add_with_targets`

:func:`constraints_clear`

:func:`constraints_copy`

:func:`copy`

:func:`flip_names`

:func:`group_add`

:func:`group_assign`

:func:`group_deselect`

:func:`group_move`

:func:`group_remove`

:func:`group_select`

:func:`group_sort`

:func:`group_unassign`

:func:`hide`

:func:`ik_add`

:func:`ik_clear`

:func:`loc_clear`

:func:`paste`

:func:`paths_calculate`

:func:`paths_clear`

:func:`paths_range_update`

:func:`paths_update`

:func:`propagate`

:func:`push`

:func:`push_rest`

:func:`quaternions_flip`

:func:`relax`

:func:`relax_rest`

:func:`reveal`

:func:`rot_clear`

:func:`rotation_mode_set`

:func:`scale_clear`

:func:`select_all`

:func:`select_constraint_target`

:func:`select_grouped`

:func:`select_hierarchy`

:func:`select_linked`

:func:`select_linked_pick`

:func:`select_mirror`

:func:`select_parent`

:func:`transforms_clear`

:func:`user_transforms_clear`

:func:`visual_transform_apply`

"""

import typing

def armature_apply(selected: bool = False) -> None:

  """

  Apply the current pose as the new rest pose

  """

  ...

def autoside_names(axis: str = 'XAXIS') -> None:

  """

  Automatically renames the selected bones according to which side of the target axis they fall on

  """

  ...

def bone_layers(layers: typing.Tuple[bool, ...] = (False, False, False, False, False, False, False, False, False, False, False, False, False, False, False, False, False, False, False, False, False, False, False, False, False, False, False, False, False, False, False, False)) -> None:

  """

  Change the layers that the selected bones belong to

  """

  ...

def breakdown(percentage: float = 0.5, prev_frame: int = 0, next_frame: int = 0, channels: str = 'ALL', axis_lock: str = 'FREE') -> None:

  """

  Create a suitable breakdown pose on the current frame

  """

  ...

def constraint_add(type: str = '') -> None:

  """

  Add a constraint to the active bone

  """

  ...

def constraint_add_with_targets(type: str = '') -> None:

  """

  Add a constraint to the active bone, with target (where applicable) set to the selected Objects/Bones

  """

  ...

def constraints_clear() -> None:

  """

  Clear all the constraints for the selected bones

  """

  ...

def constraints_copy() -> None:

  """

  Copy constraints to other selected bones

  """

  ...

def copy() -> None:

  """

  Copies the current pose of the selected bones to copy/paste buffer

  """

  ...

def flip_names(do_strip_numbers: bool = False) -> None:

  """

  Flips (and corrects) the axis suffixes of the names of selected bones

  """

  ...

def group_add() -> None:

  """

  Add a new bone group

  """

  ...

def group_assign(type: int = 0) -> None:

  """

  Add selected bones to the chosen bone group

  """

  ...

def group_deselect() -> None:

  """

  Deselect bones of active Bone Group

  """

  ...

def group_move(direction: str = 'UP') -> None:

  """

  Change position of active Bone Group in list of Bone Groups

  """

  ...

def group_remove() -> None:

  """

  Remove the active bone group

  """

  ...

def group_select() -> None:

  """

  Select bones in active Bone Group

  """

  ...

def group_sort() -> None:

  """

  Sort Bone Groups by their names in ascending order

  """

  ...

def group_unassign() -> None:

  """

  Remove selected bones from all bone groups

  """

  ...

def hide(unselected: bool = False) -> None:

  """

  Tag selected bones to not be visible in Pose Mode

  """

  ...

def ik_add(with_targets: bool = True) -> None:

  """

  Add IK Constraint to the active Bone

  """

  ...

def ik_clear() -> None:

  """

  Remove all IK Constraints from selected bones

  """

  ...

def loc_clear() -> None:

  """

  Reset locations of selected bones to their default values

  """

  ...

def paste(flipped: bool = False, selected_mask: bool = False) -> None:

  """

  Paste the stored pose on to the current pose

  """

  ...

def paths_calculate(start_frame: int = 1, end_frame: int = 250, bake_location: str = 'HEADS') -> None:

  """

  Calculate paths for the selected bones

  """

  ...

def paths_clear(only_selected: bool = False) -> None:

  """

  Clear path caches for all bones, hold Shift key for selected bones only

  """

  ...

def paths_range_update() -> None:

  """

  Update frame range for motion paths from the Scene's current frame range

  """

  ...

def paths_update() -> None:

  """

  Recalculate paths for bones that already have them

  """

  ...

def propagate(mode: str = 'WHILE_HELD', end_frame: float = 250.0) -> None:

  """

  Copy selected aspects of the current pose to subsequent poses already keyframed

  """

  ...

def push(percentage: float = 0.5, prev_frame: int = 0, next_frame: int = 0, channels: str = 'ALL', axis_lock: str = 'FREE') -> None:

  """

  Exaggerate the current pose in regards to the breakdown pose

  """

  ...

def push_rest(percentage: float = 0.5, prev_frame: int = 0, next_frame: int = 0, channels: str = 'ALL', axis_lock: str = 'FREE') -> None:

  """

  Push the current pose further away from the rest pose

  """

  ...

def quaternions_flip() -> None:

  """

  Flip quaternion values to achieve desired rotations, while maintaining the same orientations

  """

  ...

def relax(percentage: float = 0.5, prev_frame: int = 0, next_frame: int = 0, channels: str = 'ALL', axis_lock: str = 'FREE') -> None:

  """

  Make the current pose more similar to its breakdown pose

  """

  ...

def relax_rest(percentage: float = 0.5, prev_frame: int = 0, next_frame: int = 0, channels: str = 'ALL', axis_lock: str = 'FREE') -> None:

  """

  Make the current pose more similar to the rest pose

  """

  ...

def reveal(select: bool = True) -> None:

  """

  Reveal all bones hidden in Pose Mode

  """

  ...

def rot_clear() -> None:

  """

  Reset rotations of selected bones to their default values

  """

  ...

def rotation_mode_set(type: str = 'QUATERNION') -> None:

  """

  Set the rotation representation used by selected bones

  """

  ...

def scale_clear() -> None:

  """

  Reset scaling of selected bones to their default values

  """

  ...

def select_all(action: str = 'TOGGLE') -> None:

  """

  Toggle selection status of all bones

  """

  ...

def select_constraint_target() -> None:

  """

  Select bones used as targets for the currently selected bones

  """

  ...

def select_grouped(extend: bool = False, type: str = 'LAYER') -> None:

  """

  Select all visible bones grouped by similar properties

  """

  ...

def select_hierarchy(direction: str = 'PARENT', extend: bool = False) -> None:

  """

  Select immediate parent/children of selected bones

  """

  ...

def select_linked() -> None:

  """

  Select all bones linked by parent/child connections to the current selection

  """

  ...

def select_linked_pick(extend: bool = False) -> None:

  """

  Select bones linked by parent/child connections under the mouse cursor

  """

  ...

def select_mirror(only_active: bool = False, extend: bool = False) -> None:

  """

  Mirror the bone selection

  """

  ...

def select_parent() -> None:

  """

  Select bones that are parents of the currently selected bones

  """

  ...

def transforms_clear() -> None:

  """

  Reset location, rotation, and scaling of selected bones to their default values

  """

  ...

def user_transforms_clear(only_selected: bool = True) -> None:

  """

  Reset pose bone transforms to keyframed state

  """

  ...

def visual_transform_apply() -> None:

  """

  Apply final constrained position of pose bones to their transform

  """

  ...



"""


Image Buffer (imbuf)
********************

This module provides access to Blender's image manipulation API.

:func:`load`

:func:`new`

:func:`write`

"""

import typing

def load(filepath: str) -> ImBuf:

  """

  Load an image from a file.

  """

  ...

def new(size: typing.Any) -> ImBuf:

  """

  Load a new image.

  """

  ...

def write(image: ImBuf, filepath: str) -> None:

  """

  Write an image.

  """

  ...



"""


Cycles Operators
****************

:func:`add_aov`

:func:`denoise_animation`

:func:`merge_images`

:func:`remove_aov`

:func:`use_shading_nodes`

"""

import typing

def add_aov() -> None:

  """

  Add an AOV pass

  """

  ...

def denoise_animation(input_filepath: str = '', output_filepath: str = '') -> None:

  """

  Denoise rendered animation sequence using current scene and view layer settings. Requires denoising data passes and output to OpenEXR multilayer files

  """

  ...

def merge_images(input_filepath1: str = '', input_filepath2: str = '', output_filepath: str = '') -> None:

  """

  Combine OpenEXR multilayer images rendered with different sample ranges into one image with reduced noise

  """

  ...

def remove_aov() -> None:

  """

  Remove an AOV pass

  """

  ...

def use_shading_nodes() -> None:

  """

  Enable nodes on a material, world or light

  """

  ...



"""


View3D Operators
****************

:func:`background_image_add`

:func:`background_image_remove`

:func:`camera_to_view`

:func:`camera_to_view_selected`

:func:`clear_render_border`

:func:`clip_border`

:func:`copybuffer`

:func:`cursor3d`

:func:`dolly`

:func:`edit_mesh_extrude_individual_move`

:func:`edit_mesh_extrude_manifold_normal`

:func:`edit_mesh_extrude_move_normal`

:func:`edit_mesh_extrude_move_shrink_fatten`

:func:`fly`

:func:`interactive_add`

:func:`localview`

:func:`localview_remove_from`

:func:`move`

:func:`navigate`

:func:`ndof_all`

:func:`ndof_orbit`

:func:`ndof_orbit_zoom`

:func:`ndof_pan`

:func:`object_as_camera`

:func:`object_mode_pie_or_toggle`

:func:`pastebuffer`

:func:`render_border`

:func:`rotate`

:func:`ruler_add`

:func:`ruler_remove`

:func:`select`

:func:`select_box`

:func:`select_circle`

:func:`select_lasso`

:func:`select_menu`

:func:`smoothview`

:func:`snap_cursor_to_active`

:func:`snap_cursor_to_center`

:func:`snap_cursor_to_grid`

:func:`snap_cursor_to_selected`

:func:`snap_selected_to_active`

:func:`snap_selected_to_cursor`

:func:`snap_selected_to_grid`

:func:`toggle_matcap_flip`

:func:`toggle_shading`

:func:`toggle_xray`

:func:`transform_gizmo_set`

:func:`view_all`

:func:`view_axis`

:func:`view_camera`

:func:`view_center_camera`

:func:`view_center_cursor`

:func:`view_center_lock`

:func:`view_center_pick`

:func:`view_lock_clear`

:func:`view_lock_to_active`

:func:`view_orbit`

:func:`view_pan`

:func:`view_persportho`

:func:`view_roll`

:func:`view_selected`

:func:`walk`

:func:`zoom`

:func:`zoom_border`

:func:`zoom_camera_1_to_1`

"""

import typing

def background_image_add(name: str = 'Image', filepath: str = '', hide_props_region: bool = True, filter_blender: bool = False, filter_backup: bool = False, filter_image: bool = True, filter_movie: bool = True, filter_python: bool = False, filter_font: bool = False, filter_sound: bool = False, filter_text: bool = False, filter_archive: bool = False, filter_btx: bool = False, filter_collada: bool = False, filter_alembic: bool = False, filter_usd: bool = False, filter_volume: bool = False, filter_folder: bool = True, filter_blenlib: bool = False, filemode: int = 9, relative_path: bool = True, show_multiview: bool = False, use_multiview: bool = False, display_type: str = 'DEFAULT', sort_method: str = 'FILE_SORT_ALPHA') -> None:

  """

  Add a new background image

  """

  ...

def background_image_remove(index: int = 0) -> None:

  """

  Remove a background image from the 3D view

  """

  ...

def camera_to_view() -> None:

  """

  Set camera view to active view

  """

  ...

def camera_to_view_selected() -> None:

  """

  Move the camera so selected objects are framed

  """

  ...

def clear_render_border() -> None:

  """

  Clear the boundaries of the border render and disable border render

  """

  ...

def clip_border(xmin: int = 0, xmax: int = 0, ymin: int = 0, ymax: int = 0, wait_for_input: bool = True) -> None:

  """

  Set the view clipping region

  """

  ...

def copybuffer() -> None:

  """

  Selected objects are copied to the clipboard

  """

  ...

def cursor3d(use_depth: bool = True, orientation: str = 'VIEW') -> None:

  """

  Set the location of the 3D cursor

  """

  ...

def dolly(mx: int = 0, my: int = 0, delta: int = 0, use_cursor_init: bool = True) -> None:

  """

  Dolly in/out in the view

  """

  ...

def edit_mesh_extrude_individual_move() -> None:

  """

  Extrude each individual face separately along local normals

  """

  ...

def edit_mesh_extrude_manifold_normal() -> None:

  """

  Extrude manifold region along normals

  """

  ...

def edit_mesh_extrude_move_normal(dissolve_and_intersect: bool = False) -> None:

  """

  Extrude region together along the average normal

  """

  ...

def edit_mesh_extrude_move_shrink_fatten() -> None:

  """

  Extrude region together along local normals

  """

  ...

def fly() -> None:

  """

  Interactively fly around the scene

  """

  ...

def interactive_add(primitive_type: str = 'CUBE', plane_axis: str = 'Z', plane_depth: str = 'SURFACE', plane_origin: str = 'BASE', plane_orientation: str = 'SURFACE', wait_for_input: bool = True) -> None:

  """

  Interactively add an object

  """

  ...

def localview(frame_selected: bool = True) -> None:

  """

  Toggle display of selected object(s) separately and centered in view

  """

  ...

def localview_remove_from() -> None:

  """

  Move selected objects out of local view

  """

  ...

def move(use_cursor_init: bool = True) -> None:

  """

  Move the view

  """

  ...

def navigate() -> None:

  """

  Interactively navigate around the scene (uses the mode (walk/fly) preference)

  """

  ...

def ndof_all() -> None:

  """

  Pan and rotate the view with the 3D mouse

  """

  ...

def ndof_orbit() -> None:

  """

  Orbit the view using the 3D mouse

  """

  ...

def ndof_orbit_zoom() -> None:

  """

  Orbit and zoom the view using the 3D mouse

  """

  ...

def ndof_pan() -> None:

  """

  Pan the view with the 3D mouse

  """

  ...

def object_as_camera() -> None:

  """

  Set the active object as the active camera for this view or scene

  """

  ...

def object_mode_pie_or_toggle() -> None:

  """

  Undocumented, consider `contributing <https://developer.blender.org/T51061>`_.

  """

  ...

def pastebuffer(autoselect: bool = True, active_collection: bool = True) -> None:

  """

  Objects from the clipboard are pasted

  """

  ...

def render_border(xmin: int = 0, xmax: int = 0, ymin: int = 0, ymax: int = 0, wait_for_input: bool = True) -> None:

  """

  Set the boundaries of the border render and enable border render

  """

  ...

def rotate(use_cursor_init: bool = True) -> None:

  """

  Rotate the view

  """

  ...

def ruler_add() -> None:

  """

  Add ruler

  """

  ...

def ruler_remove() -> None:

  """

  Undocumented, consider `contributing <https://developer.blender.org/T51061>`_.

  """

  ...

def select(extend: bool = False, deselect: bool = False, toggle: bool = False, deselect_all: bool = False, center: bool = False, enumerate: bool = False, object: bool = False, location: typing.Tuple[int, int] = (0, 0)) -> None:

  """

  Select and activate item(s)

  """

  ...

def select_box(xmin: int = 0, xmax: int = 0, ymin: int = 0, ymax: int = 0, wait_for_input: bool = True, mode: str = 'SET') -> None:

  """

  Select items using box selection

  """

  ...

def select_circle(x: int = 0, y: int = 0, radius: int = 25, wait_for_input: bool = True, mode: str = 'SET') -> None:

  """

  Select items using circle selection

  """

  ...

def select_lasso(path: typing.Union[typing.Sequence[OperatorMousePath], typing.Mapping[str, OperatorMousePath], bpy.types.bpy_prop_collection] = None, mode: str = 'SET') -> None:

  """

  Select items using lasso selection

  """

  ...

def select_menu(name: str = '', extend: bool = False, deselect: bool = False, toggle: bool = False) -> None:

  """

  Menu object selection

  """

  ...

def smoothview() -> None:

  """

  Undocumented, consider `contributing <https://developer.blender.org/T51061>`_.

  """

  ...

def snap_cursor_to_active() -> None:

  """

  Snap 3D cursor to the active item

  """

  ...

def snap_cursor_to_center() -> None:

  """

  Snap 3D cursor to the world origin

  """

  ...

def snap_cursor_to_grid() -> None:

  """

  Snap 3D cursor to the nearest grid division

  """

  ...

def snap_cursor_to_selected() -> None:

  """

  Snap 3D cursor to the middle of the selected item(s)

  """

  ...

def snap_selected_to_active() -> None:

  """

  Snap selected item(s) to the active item

  """

  ...

def snap_selected_to_cursor(use_offset: bool = True) -> None:

  """

  Snap selected item(s) to the 3D cursor

  """

  ...

def snap_selected_to_grid() -> None:

  """

  Snap selected item(s) to their nearest grid division

  """

  ...

def toggle_matcap_flip() -> None:

  """

  Flip MatCap

  """

  ...

def toggle_shading(type: str = 'WIREFRAME') -> None:

  """

  Toggle shading type in 3D viewport

  """

  ...

def toggle_xray() -> None:

  """

  Transparent scene display. Allow selecting through items

  """

  ...

def transform_gizmo_set(extend: bool = False, type: typing.Set[str] = {}) -> None:

  """

  Set the current transform gizmo

  """

  ...

def view_all(use_all_regions: bool = False, center: bool = False) -> None:

  """

  View all objects in scene

  """

  ...

def view_axis(type: str = 'LEFT', align_active: bool = False, relative: bool = False) -> None:

  """

  Use a preset viewpoint

  """

  ...

def view_camera() -> None:

  """

  Toggle the camera view

  """

  ...

def view_center_camera() -> None:

  """

  Center the camera view, resizing the view to fit its bounds

  """

  ...

def view_center_cursor() -> None:

  """

  Center the view so that the cursor is in the middle of the view

  """

  ...

def view_center_lock() -> None:

  """

  Center the view lock offset

  """

  ...

def view_center_pick() -> None:

  """

  Center the view to the Z-depth position under the mouse cursor

  """

  ...

def view_lock_clear() -> None:

  """

  Clear all view locking

  """

  ...

def view_lock_to_active() -> None:

  """

  Lock the view to the active object/bone

  """

  ...

def view_orbit(angle: float = 0.0, type: str = 'ORBITLEFT') -> None:

  """

  Orbit the view

  """

  ...

def view_pan(type: str = 'PANLEFT') -> None:

  """

  Pan the view in a given direction

  """

  ...

def view_persportho() -> None:

  """

  Switch the current view from perspective/orthographic projection

  """

  ...

def view_roll(angle: float = 0.0, type: str = 'ANGLE') -> None:

  """

  Roll the view

  """

  ...

def view_selected(use_all_regions: bool = False) -> None:

  """

  Move the view to the selection center

  """

  ...

def walk() -> None:

  """

  Interactively walk around the scene

  """

  ...

def zoom(mx: int = 0, my: int = 0, delta: int = 0, use_cursor_init: bool = True) -> None:

  """

  Zoom in/out in the view

  """

  ...

def zoom_border(xmin: int = 0, xmax: int = 0, ymin: int = 0, ymax: int = 0, wait_for_input: bool = True, zoom_out: bool = False) -> None:

  """

  Zoom in the view to the nearest object contained in the border

  """

  ...

def zoom_camera_1_to_1() -> None:

  """

  Match the camera to 1:1 to the render output

  """

  ...



"""


Sound Operators
***************

:func:`bake_animation`

:func:`mixdown`

:func:`open`

:func:`open_mono`

:func:`pack`

:func:`unpack`

:func:`update_animation_flags`

"""

import typing

def bake_animation() -> None:

  """

  Update the audio animation cache

  """

  ...

def mixdown(filepath: str = '', check_existing: bool = True, filter_blender: bool = False, filter_backup: bool = False, filter_image: bool = False, filter_movie: bool = False, filter_python: bool = False, filter_font: bool = False, filter_sound: bool = True, filter_text: bool = False, filter_archive: bool = False, filter_btx: bool = False, filter_collada: bool = False, filter_alembic: bool = False, filter_usd: bool = False, filter_volume: bool = False, filter_folder: bool = True, filter_blenlib: bool = False, filemode: int = 9, relative_path: bool = True, display_type: str = 'DEFAULT', sort_method: str = 'FILE_SORT_ALPHA', accuracy: int = 1024, container: str = 'FLAC', codec: str = 'FLAC', format: str = 'S16', bitrate: int = 192, split_channels: bool = False) -> None:

  """

  Mix the scene's audio to a sound file

  """

  ...

def open(filepath: str = '', hide_props_region: bool = True, filter_blender: bool = False, filter_backup: bool = False, filter_image: bool = False, filter_movie: bool = True, filter_python: bool = False, filter_font: bool = False, filter_sound: bool = True, filter_text: bool = False, filter_archive: bool = False, filter_btx: bool = False, filter_collada: bool = False, filter_alembic: bool = False, filter_usd: bool = False, filter_volume: bool = False, filter_folder: bool = True, filter_blenlib: bool = False, filemode: int = 9, relative_path: bool = True, show_multiview: bool = False, use_multiview: bool = False, display_type: str = 'DEFAULT', sort_method: str = 'FILE_SORT_ALPHA', cache: bool = False, mono: bool = False) -> None:

  """

  Load a sound file

  """

  ...

def open_mono(filepath: str = '', hide_props_region: bool = True, filter_blender: bool = False, filter_backup: bool = False, filter_image: bool = False, filter_movie: bool = True, filter_python: bool = False, filter_font: bool = False, filter_sound: bool = True, filter_text: bool = False, filter_archive: bool = False, filter_btx: bool = False, filter_collada: bool = False, filter_alembic: bool = False, filter_usd: bool = False, filter_volume: bool = False, filter_folder: bool = True, filter_blenlib: bool = False, filemode: int = 9, relative_path: bool = True, show_multiview: bool = False, use_multiview: bool = False, display_type: str = 'DEFAULT', sort_method: str = 'FILE_SORT_ALPHA', cache: bool = False, mono: bool = True) -> None:

  """

  Load a sound file as mono

  """

  ...

def pack() -> None:

  """

  Pack the sound into the current blend file

  """

  ...

def unpack(method: str = 'USE_LOCAL', id: str = '') -> None:

  """

  Unpack the sound to the samples filename

  """

  ...

def update_animation_flags() -> None:

  """

  Update animation flags

  """

  ...



"""


Workspace Operators
*******************

:func:`add`

:func:`append_activate`

:func:`delete`

:func:`duplicate`

:func:`reorder_to_back`

:func:`reorder_to_front`

"""

import typing

def add() -> None:

  """

  Add a new workspace by duplicating the current one or appending one from the user configuration

  """

  ...

def append_activate(idname: str = '', filepath: str = '') -> None:

  """

  Append a workspace and make it the active one in the current window

  """

  ...

def delete() -> None:

  """

  Delete the active workspace

  """

  ...

def duplicate() -> None:

  """

  Add a new workspace

  """

  ...

def reorder_to_back() -> None:

  """

  Reorder workspace to be first in the list

  """

  ...

def reorder_to_front() -> None:

  """

  Reorder workspace to be first in the list

  """

  ...



"""


GPU Types (gpu.types)
*********************

:class:`GPUBatch`

:class:`GPUIndexBuf`

:class:`GPUOffScreen`

:class:`GPUShader`

:class:`GPUVertBuf`

:class:`GPUVertFormat`

"""

import typing

import mathutils

import bpy

class GPUBatch:

  """

  Reusable container for drawable geometry.

  """

  def __init__(self, type: str, buf: GPUVertBuf, elem: GPUIndexBuf = None) -> None:

    """

    :arg type:        
      One of these primitive types: {
*POINTS*,
*LINES*,
*TRIS*,
*LINE_STRIP*,
*LINE_LOOP*,
*TRI_STRIP*,
*TRI_FAN*,
*LINES_ADJ*,
*TRIS_ADJ*,
*LINE_STRIP_ADJ* }

    :type type:       
      *str*

    :arg buf:         
      Vertex buffer containing all or some of the attributes required for drawing.

    :type buf:        
      :class:`gpu.types.GPUVertBuf`

    :arg elem:        
      An optional index buffer.

    :type elem:       
      :class:`gpu.types.GPUIndexBuf`

    """

    ...

  def draw(self, program: GPUShader = None) -> None:

    """

    Run the drawing program with the parameters assigned to the batch.

    """

    ...

  def program_set(self, program: GPUShader) -> None:

    """

    Assign a shader to this batch that will be used for drawing when not overwritten later.
Note: This method has to be called in the draw context that the batch will be drawn in.
This function does not need to be called when you always
set the shader when calling :meth:`gpu.types.GPUBatch.draw`.

    """

    ...

  def vertbuf_add(self, buf: GPUVertBuf) -> None:

    """

    Add another vertex buffer to the Batch.
It is not possible to add more vertices to the batch using this method.
Instead it can be used to add more attributes to the existing vertices.
A good use case would be when you have a separate
vertex buffer for vertex positions and vertex normals.
Current a batch can have at most 6 vertex buffers.

    """

    ...

class GPUIndexBuf:

  """

  Contains an index buffer.

  """

  def __init__(self, type: str, seq: typing.Any) -> None:

    """

    :param type:      
      One of these primitive types: {
*POINTS*,
*LINES*,
*TRIS*,
*LINE_STRIP_ADJ* }

    :type type:       
      *str*

    :param seq:       
      Indices this index buffer will contain.
Whether a 1D or 2D sequence is required depends on the type.
Optionally the sequence can support the buffer protocol.

    :type seq:        
      1D or 2D sequence

    """

    ...

class GPUOffScreen:

  """

  This object gives access to off screen buffers.

  """

  def __init__(self, width: int, height: int) -> None:

    """

    :arg width:       
      Horizontal dimension of the buffer.

    :type width:      
      *int*

    :arg height:      
      Vertical dimension of the buffer.

    :type height:     
      *int*

    """

    ...

  def bind(self, save: bool = True) -> None:

    """

    Bind the offscreen object.
To make sure that the offscreen gets unbind whether an exception occurs or not,
pack it into a *with* statement.

    """

    ...

  def draw_view3d(self, scene: bpy.types.Scene, view_layer: bpy.types.ViewLayer, view3d: bpy.types.SpaceView3D, region: bpy.types.Region, view_matrix: mathutils.Matrix, projection_matrix: mathutils.Matrix) -> None:

    """

    Draw the 3d viewport in the offscreen object.

    """

    ...

  def free(self) -> None:

    """

    Free the offscreen object.
The framebuffer, texture and render objects will no longer be accessible.

    """

    ...

  def unbind(self, restore: bool = True) -> None:

    """

    Unbind the offscreen object.

    """

    ...

  color_texture: int = ...

  """

  OpenGL bindcode for the color texture.

  """

  height: int = ...

  """

  Height of the texture.

  """

  width: int = ...

  """

  Width of the texture.

  """

class GPUShader:

  """

  GPUShader combines multiple GLSL shaders into a program used for drawing.
It must contain a vertex and fragment shaders, with an optional geometry shader.

  The GLSL ``#version`` directive is automatically included at the top of shaders,
and set to 330. Some preprocessor directives are automatically added according to
the Operating System or availability: ``GPU_ATI``, ``GPU_NVIDIA`` and ``GPU_INTEL``.

  The following extensions are enabled by default if supported by the GPU:
``GL_ARB_texture_gather``, ``GL_ARB_texture_cube_map_array``
and ``GL_ARB_shader_draw_parameters``.

  To debug shaders, use the ``--debug-gpu-shaders`` command line option
to see full GLSL shader compilation and linking errors.

  For drawing user interface elements and gizmos, use
``fragOutput = blender_srgb_to_framebuffer_space(fragOutput)``
to transform the output sRGB colors to the frame-buffer color-space.

  """

  def __init__(self, vertexcode: str, fragcode: typing.Any, geocode: typing.Any = None, libcode: typing.Any = None, defines: typing.Any = None) -> None:

    """

    :param vertexcode:
      Vertex shader code.

    :type vertexcode: 
      str

    :param fragcode:  
      Fragment shader code.

    :type value:      
      str

    :param geocode:   
      Geometry shader code.

    :type value:      
      str

    :param libcode:   
      Code with functions and presets to be shared between shaders.

    :type value:      
      str

    :param defines:   
      Preprocessor directives.

    :type value:      
      str

    """

    ...

  def attr_from_name(self, name: str) -> int:

    """

    Get attribute location by name.

    """

    ...

  def bind(self) -> None:

    """

    Bind the shader object. Required to be able to change uniforms of this shader.

    """

    ...

  def calc_format(self) -> typing.Any:

    """

    Build a new format based on the attributes of the shader.

    """

    ...

  def uniform_block_from_name(self, name: str) -> int:

    """

    Get uniform block location by name.

    """

    ...

  def uniform_bool(self, name: str, seq: typing.Sequence[bool]) -> None:

    """

    Specify the value of a uniform variable for the current program object.

    """

    ...

  def uniform_float(self, name: str, value: typing.Any) -> None:

    """

    Specify the value of a uniform variable for the current program object.

    """

    ...

  def uniform_from_name(self, name: str) -> int:

    """

    Get uniform location by name.

    """

    ...

  def uniform_int(self, name: str, seq: typing.Sequence[typing.Any]) -> None:

    """

    Specify the value of a uniform variable for the current program object.

    """

    ...

  def uniform_vector_float(self, location: int, buffer: typing.Sequence[float], length: int, count: int) -> None:

    """

    Set the buffer to fill the uniform.

    """

    ...

  def uniform_vector_int(self, location: typing.Any, buffer: typing.Any, length: typing.Any, count: typing.Any) -> None:

    """

    See GPUShader.uniform_vector_float(...) description.

    """

    ...

  program: int = ...

  """

  The name of the program object for use by the OpenGL API (read-only).

  """

class GPUVertBuf:

  """

  Contains a VBO.

  """

  def __init__(self, len: typing.Any, format: typing.Any) -> None:

    """

    :param len:       
      Amount of vertices that will fit into this buffer.

    :type type:       
      *int*

    :param format:    
      Vertex format.

    :type buf:        
      :class:`gpu.types.GPUVertFormat`

    """

    ...

  def attr_fill(self, id: typing.Union[int, str], data: typing.Sequence[typing.Any]) -> None:

    """

    Insert data into the buffer for a single attribute.

    """

    ...

class GPUVertFormat:

  """

  This object contains information about the structure of a vertex buffer.

  """

  def attr_add(self, id: str, comp_type: str, len: int, fetch_mode: str) -> None:

    """

    Add a new attribute to the format.

    """

    ...



"""


Geometry Operators
******************

:func:`attribute_add`

:func:`attribute_remove`

"""

import typing

def attribute_add(name: str = 'Attribute', data_type: str = 'FLOAT', domain: str = 'POINT') -> None:

  """

  Add attribute to geometry

  """

  ...

def attribute_remove() -> None:

  """

  Remove attribute from geometry

  """

  ...



"""


Utilities (bpy.utils)
*********************

This module contains utility functions specific to blender but
not associated with blenders internal data.

:func:`blend_paths`

:func:`escape_identifier`

:func:`register_class`

:func:`resource_path`

:func:`unregister_class`

:func:`keyconfig_init`

:func:`keyconfig_set`

:func:`load_scripts`

:func:`modules_from_path`

:func:`preset_find`

:func:`preset_paths`

:func:`refresh_script_paths`

:func:`app_template_paths`

:func:`register_manual_map`

:func:`unregister_manual_map`

:func:`register_classes_factory`

:func:`register_submodule_factory`

:func:`register_tool`

:func:`make_rna_paths`

:func:`manual_map`

:func:`script_path_user`

:func:`script_path_pref`

:func:`script_paths`

:func:`smpte_from_frame`

:func:`smpte_from_seconds`

:func:`unregister_tool`

:func:`user_resource`

:func:`execfile`

"""

from . import units

from . import previews

import typing

def blend_paths(absolute: bool = False, packed: bool = False, local: bool = False) -> typing.List[str]:

  """

  Returns a list of paths to external files referenced by the loaded .blend file.

  """

  ...

def escape_identifier(string: str) -> str:

  """

  Simple string escaping function used for animation paths.

  """

  ...

def register_class(self, cls: typing.Type) -> None:

  """

  Register a subclass of a Blender type class.

  Note: If the class has a *register* class method it will be called
before registration.

  """

  ...

def resource_path(type: str, major: int = bpy.app.version[0], minor: str = bpy.app.version[1]) -> str:

  """

  Return the base path for storing system files.

  """

  ...

def unregister_class(self, cls: typing.Any) -> None:

  """

  Unload the Python class from blender.

  If the class has an *unregister* class method it will be called
before unregistering.

  """

  ...

def keyconfig_init() -> None:

  ...

def keyconfig_set(filepath: typing.Any, report: typing.Any = None) -> None:

  ...

def load_scripts(reload_scripts: bool = False, refresh_scripts: bool = False) -> None:

  """

  Load scripts and run each modules register function.

  """

  ...

def modules_from_path(path: str, loaded_modules: typing.Set[typing.Any]) -> typing.List[typing.Any]:

  """

  Load all modules in a path and return them as a list.

  """

  ...

def preset_find(name: typing.Any, preset_path: typing.Any, display_name: typing.Any = False, ext: typing.Any = '.py') -> None:

  ...

def preset_paths(subdir: str) -> typing.List[typing.Any]:

  """

  Returns a list of paths for a specific preset.

  """

  ...

def refresh_script_paths() -> None:

  """

  Run this after creating new script paths to update sys.path

  """

  ...

def app_template_paths(subdir: str = None) -> typing.Any:

  """

  Returns valid application template paths.

  """

  ...

def register_manual_map(manual_hook: typing.Any) -> None:

  ...

def unregister_manual_map(manual_hook: typing.Any) -> None:

  ...

def register_classes_factory(classes: typing.Any) -> None:

  """

  Utility function to create register and unregister functions
which simply registers and unregisters a sequence of classes.

  """

  ...

def register_submodule_factory(module_name: str, submodule_names: typing.List[str]) -> typing.Tuple[typing.Any, ...]:

  """

  Utility function to create register and unregister functions
which simply load submodules,
calling their register & unregister functions.

  Note: Modules are registered in the order given,
unregistered in reverse order.

  """

  ...

def register_tool(tool_cls: typing.Any) -> None:

  """

  Register a tool in the toolbar.

  """

  ...

def make_rna_paths(struct_name: str, prop_name: str, enum_name: str) -> typing.Tuple[str, ...]:

  """

  Create RNA "paths" from given names.

  """

  ...

def manual_map() -> None:

  ...

def script_path_user() -> None:

  """

  returns the env var and falls back to home dir or None

  """

  ...

def script_path_pref() -> None:

  """

  returns the user preference or None

  """

  ...

def script_paths(subdir: str = None, user_pref: bool = True, check_all: bool = False, use_user: typing.Any = True) -> typing.List[typing.Any]:

  """

  Returns a list of valid script paths.

  """

  ...

def smpte_from_frame(frame: typing.Union[int, float], fps: typing.Any = None, fps_base: typing.Any = None) -> str:

  """

  Returns an SMPTE formatted string from the *frame*:
``HH:MM:SS:FF``.

  If *fps* and *fps_base* are not given the current scene is used.

  """

  ...

def smpte_from_seconds(time: int, fps: typing.Any = None, fps_base: typing.Any = None) -> str:

  """

  Returns an SMPTE formatted string from the *time*:
``HH:MM:SS:FF``.

  If *fps* and *fps_base* are not given the current scene is used.

  """

  ...

def unregister_tool(tool_cls: typing.Any) -> None:

  ...

def user_resource(resource_type: typing.Any, path: typing.Any = '', create: bool = False) -> str:

  """

  Return a user resource path (normally from the users home directory).

  """

  ...

def execfile(filepath: typing.Any, mod: typing.Any = None) -> None:

  ...



"""


Import Anim Operators
*********************

:func:`bvh`

"""

import typing

def bvh(filepath: str = '', filter_glob: str = '*.bvh', target: str = 'ARMATURE', global_scale: float = 1.0, frame_start: int = 1, use_fps_scale: bool = False, update_scene_fps: bool = False, update_scene_duration: bool = False, use_cyclic: bool = False, rotate_mode: str = 'NATIVE', axis_forward: str = '-Z', axis_up: str = 'Y') -> None:

  """

  Load a BVH motion capture file

  """

  ...



"""


Marker Operators
****************

:func:`add`

:func:`camera_bind`

:func:`delete`

:func:`duplicate`

:func:`make_links_scene`

:func:`move`

:func:`rename`

:func:`select`

:func:`select_all`

:func:`select_box`

"""

import typing

def add() -> None:

  """

  Add a new time marker

  """

  ...

def camera_bind() -> None:

  """

  Bind the selected camera to a marker on the current frame

  """

  ...

def delete() -> None:

  """

  Delete selected time marker(s)

  """

  ...

def duplicate(frames: int = 0) -> None:

  """

  Duplicate selected time marker(s)

  """

  ...

def make_links_scene(scene: str = '') -> None:

  """

  Copy selected markers to another scene

  """

  ...

def move(frames: int = 0, tweak: bool = False) -> None:

  """

  Move selected time marker(s)

  """

  ...

def rename(name: str = 'RenamedMarker') -> None:

  """

  Rename first selected time marker

  """

  ...

def select(wait_to_deselect_others: bool = False, mouse_x: int = 0, mouse_y: int = 0, extend: bool = False, camera: bool = False) -> None:

  """

  Select time marker(s)

  """

  ...

def select_all(action: str = 'TOGGLE') -> None:

  """

  Change selection of all time markers

  """

  ...

def select_box(xmin: int = 0, xmax: int = 0, ymin: int = 0, ymax: int = 0, wait_for_input: bool = True, mode: str = 'SET', tweak: bool = False) -> None:

  """

  Select all time markers using box selection

  """

  ...



"""


Poselib Operators
*****************

:func:`action_sanitize`

:func:`apply_pose`

:func:`browse_interactive`

:func:`new`

:func:`pose_add`

:func:`pose_move`

:func:`pose_remove`

:func:`pose_rename`

:func:`unlink`

"""

import typing

def action_sanitize() -> None:

  """

  Make action suitable for use as a Pose Library

  """

  ...

def apply_pose(pose_index: int = -1) -> None:

  """

  Apply specified Pose Library pose to the rig

  """

  ...

def browse_interactive(pose_index: int = -1) -> None:

  """

  Interactively browse poses in 3D-View

  """

  ...

def new() -> None:

  """

  Add New Pose Library to active Object

  """

  ...

def pose_add(frame: int = 1, name: str = 'Pose') -> None:

  """

  Add the current Pose to the active Pose Library

  """

  ...

def pose_move(pose: str = '', direction: str = 'UP') -> None:

  """

  Move the pose up or down in the active Pose Library

  """

  ...

def pose_remove(pose: str = '') -> None:

  """

  Remove nth pose from the active Pose Library

  """

  ...

def pose_rename(name: str = 'RenamedPose', pose: str = '') -> None:

  """

  Rename specified pose from the active Pose Library

  """

  ...

def unlink() -> None:

  """

  Remove Pose Library from active Object

  """

  ...



"""


GPU Select (gpu.select)
***********************

This module provides access to selection.

:func:`load_id`

"""

import typing

def load_id(id: typing.Any) -> None:

  """

  Set the selection ID.

  """

  ...



"""


Application Data (bpy.app)
**************************

This module contains application values that remain unchanged during runtime.

:data:`autoexec_fail`

:data:`autoexec_fail_message`

:data:`autoexec_fail_quiet`

:data:`binary_path_python`

:data:`debug`

:data:`debug_depsgraph`

:data:`debug_depsgraph_build`

:data:`debug_depsgraph_eval`

:data:`debug_depsgraph_pretty`

:data:`debug_depsgraph_tag`

:data:`debug_depsgraph_time`

:data:`debug_events`

:data:`debug_ffmpeg`

:data:`debug_freestyle`

:data:`debug_gpumem`

:data:`debug_handlers`

:data:`debug_io`

:data:`debug_python`

:data:`debug_simdata`

:data:`debug_value`

:data:`debug_wm`

:data:`driver_namespace`

:data:`render_icon_size`

:data:`render_preview_size`

:data:`tempdir`

:data:`use_event_simulate`

:data:`use_userpref_skip_save_on_exit`

:data:`background`

:data:`factory_startup`

:data:`build_branch`

:data:`build_cflags`

:data:`build_commit_date`

:data:`build_commit_time`

:data:`build_cxxflags`

:data:`build_date`

:data:`build_hash`

:data:`build_linkflags`

:data:`build_platform`

:data:`build_system`

:data:`build_time`

:data:`build_type`

:data:`build_commit_timestamp`

:data:`binary_path`

:data:`version_char`

:data:`version_cycle`

:data:`version_string`

:data:`version`

:data:`alembic`

:data:`build_options`

:data:`ffmpeg`

:data:`ocio`

:data:`oiio`

:data:`opensubdiv`

:data:`openvdb`

:data:`sdl`

:data:`usd`

"""

from . import translations

from . import timers

from . import icons

from . import handlers

import typing

autoexec_fail: typing.Any = ...

"""

Undocumented, consider `contributing <https://developer.blender.org/T51061>`_.

"""

autoexec_fail_message: typing.Any = ...

"""

Undocumented, consider `contributing <https://developer.blender.org/T51061>`_.

"""

autoexec_fail_quiet: typing.Any = ...

"""

Undocumented, consider `contributing <https://developer.blender.org/T51061>`_.

"""

binary_path_python: typing.Any = ...

"""

String, the path to the python executable (read-only). Deprecated! Use ``sys.executable`` instead.

"""

debug: typing.Any = ...

"""

Boolean, for debug info (started with --debug / --debug_* matching this attribute name)

"""

debug_depsgraph: typing.Any = ...

"""

Boolean, for debug info (started with --debug / --debug_* matching this attribute name)

"""

debug_depsgraph_build: typing.Any = ...

"""

Boolean, for debug info (started with --debug / --debug_* matching this attribute name)

"""

debug_depsgraph_eval: typing.Any = ...

"""

Boolean, for debug info (started with --debug / --debug_* matching this attribute name)

"""

debug_depsgraph_pretty: typing.Any = ...

"""

Boolean, for debug info (started with --debug / --debug_* matching this attribute name)

"""

debug_depsgraph_tag: typing.Any = ...

"""

Boolean, for debug info (started with --debug / --debug_* matching this attribute name)

"""

debug_depsgraph_time: typing.Any = ...

"""

Boolean, for debug info (started with --debug / --debug_* matching this attribute name)

"""

debug_events: typing.Any = ...

"""

Boolean, for debug info (started with --debug / --debug_* matching this attribute name)

"""

debug_ffmpeg: typing.Any = ...

"""

Boolean, for debug info (started with --debug / --debug_* matching this attribute name)

"""

debug_freestyle: typing.Any = ...

"""

Boolean, for debug info (started with --debug / --debug_* matching this attribute name)

"""

debug_gpumem: typing.Any = ...

"""

Boolean, for debug info (started with --debug / --debug_* matching this attribute name)

"""

debug_handlers: typing.Any = ...

"""

Boolean, for debug info (started with --debug / --debug_* matching this attribute name)

"""

debug_io: typing.Any = ...

"""

Boolean, for debug info (started with --debug / --debug_* matching this attribute name)

"""

debug_python: typing.Any = ...

"""

Boolean, for debug info (started with --debug / --debug_* matching this attribute name)

"""

debug_simdata: typing.Any = ...

"""

Boolean, for debug info (started with --debug / --debug_* matching this attribute name)

"""

debug_value: typing.Any = ...

"""

Short, number which can be set to non-zero values for testing purposes

"""

debug_wm: typing.Any = ...

"""

Boolean, for debug info (started with --debug / --debug_* matching this attribute name)

"""

driver_namespace: typing.Any = ...

"""

Dictionary for drivers namespace, editable in-place, reset on file load (read-only)

"""

render_icon_size: typing.Any = ...

"""

Reference size for icon/preview renders (read-only)

"""

render_preview_size: typing.Any = ...

"""

Reference size for icon/preview renders (read-only)

"""

tempdir: typing.Any = ...

"""

String, the temp directory used by blender (read-only)

"""

use_event_simulate: typing.Any = ...

"""

Boolean, for application behavior (started with --enable-* matching this attribute name)

"""

use_userpref_skip_save_on_exit: typing.Any = ...

"""

Boolean, for application behavior (started with --enable-* matching this attribute name)

"""

background: typing.Any = ...

"""

Boolean, True when blender is running without a user interface (started with -b)

"""

factory_startup: typing.Any = ...

"""

Boolean, True when blender is running with --factory-startup)

"""

build_branch: typing.Any = ...

"""

The branch this blender instance was built from

"""

build_cflags: typing.Any = ...

"""

C compiler flags

"""

build_commit_date: typing.Any = ...

"""

The date of commit this blender instance was built

"""

build_commit_time: typing.Any = ...

"""

The time of commit this blender instance was built

"""

build_cxxflags: typing.Any = ...

"""

C++ compiler flags

"""

build_date: typing.Any = ...

"""

The date this blender instance was built

"""

build_hash: typing.Any = ...

"""

The commit hash this blender instance was built with

"""

build_linkflags: typing.Any = ...

"""

Binary linking flags

"""

build_platform: typing.Any = ...

"""

The platform this blender instance was built for

"""

build_system: typing.Any = ...

"""

Build system used

"""

build_time: typing.Any = ...

"""

The time this blender instance was built

"""

build_type: typing.Any = ...

"""

The type of build (Release, Debug)

"""

build_commit_timestamp: typing.Any = ...

"""

The unix timestamp of commit this blender instance was built

"""

binary_path: typing.Any = ...

"""

The location of Blender's executable, useful for utilities that open new instances

"""

version_char: typing.Any = ...

"""

Deprecated, always an empty string

"""

version_cycle: typing.Any = ...

"""

The release status of this build alpha/beta/rc/release

"""

version_string: typing.Any = ...

"""

The Blender version formatted as a string

"""

version: typing.Any = ...

"""

The Blender version as a tuple of 3 numbers. eg. (2, 83, 1)

"""

alembic: typing.Any = ...

"""

constant value bpy.app.alembic(supported=True, version=(1, 7, 12), version_string=' 1,  7, 12')

"""

build_options: typing.Any = ...

"""

constant value bpy.app.build_options(bullet=True, codec_avi=True, codec_ffmpeg=True, codec_sndfile=True, compositor=True, cycles=True, cycles_osl=True, freestyle=True, image_cineon=True, image_dds=True, image_hdr=True, image_openexr=True, image_openjpeg=True, image_tiff=True, input_ndof=True, audaspace=True, international=True, openal=True, opensubdiv=True, sdl=True, sdl_dynload=True, jack=True, libmv=True, mod_oceansim=True, mod_remesh=True, collada=True, opencolorio=True, openmp=False, openvdb=True, alembic=True, ...)

"""

ffmpeg: typing.Any = ...

"""

constant value bpy.app.ffmpeg(supported=True, avcodec_version=(58, 54, 100), avcodec_version_string='58, 54, 100', avdevice_version=(58, 8, 100), avdevice_version_string='58,  8, 100', avformat_version=(58, 29, 100), avformat_version_string='58, 29, 100', avutil_version=(56, 31, 100), avutil_version_string='56, 31, 100', swscale_version=(5, 5, 100), swscale_version_string=' 5,  5, 100')

"""

ocio: typing.Any = ...

"""

constant value bpy.app.ocio(supported=True, version=(1, 1, 1), version_string=' 1,  1,  1')

"""

oiio: typing.Any = ...

"""

constant value bpy.app.oiio(supported=True, version=(2, 1, 15), version_string=' 2,  1, 15')

"""

opensubdiv: typing.Any = ...

"""

constant value bpy.app.opensubdiv(supported=True, version=(0, 0, 0), version_string=' 0,  0,  0')

"""

openvdb: typing.Any = ...

"""

constant value bpy.app.openvdb(supported=True, version=(7, 0, 0), version_string=' 7,  0,  0')

"""

sdl: typing.Any = ...

"""

constant value bpy.app.sdl(supported=True, version=(2, 0, 10), version_string='2.0.10', available=True)

"""

usd: typing.Any = ...

"""

constant value bpy.app.usd(supported=True, version=(0, 20, 5), version_string=' 0, 20,  5')

"""
